/*****************************************************************************
* This JavaScript file contains all functions relating to close all.
****************************************************************************/

   var CAForm = "";     //The form in the hidden iframe

// ****************************************************************************
// setForm
//
// Sets the value for the CAForm variable.
// Depending on whether close all was called from the Desktop or from an application
// page, the way in which the form in the hidden iframe is accessed varies.
// ****************************************************************************
function setForm() {
   closeAllDebug("setForm()");
   if (isDesktop) {
      CAForm = window.frames['closeAllIframe'].document.forms["closeAllForm"];
   }
   else {
      CAForm = window.opener.frames['closeAllIframe'].document.forms["closeAllForm"];
   }
}

// ****************************************************************************
// closeAll
//
// Initial function called by invoking close all from the context menu. Sets the
// values of the hidden fields on the form within the hidden iframe accordingly
// and submits the form.
// ****************************************************************************
function closeAll() {
   closeAllDebug("closeAll()");

   setForm();
   setHiddenFields();

   closeAllDebug("setting closeAllMechanism to close_all_check_only");
   CAForm.elements["HAFOperator.closeAllMechanism"].value = "close_all_check_only";
   closeAllDebug("submitting form")
   CAForm.submit();
   waitForCloseAllFinish();
}

// ****************************************************************************
// waitForCloseAllFinish
//
// Timer function used to ensure that the processing for close all has finished.
// ****************************************************************************
function waitForCloseAllFinish() {
   setForm();
   try {
      closeAllDebug("waitForCloseAllFinish(), try, CAForm=" + CAForm + ", value=" + CAForm.elements["HAFOperator.closeAllFinished"].value);
      finished = CAForm.elements["HAFOperator.closeAllFinished"].value;
   }
   catch (ex) {
      closeAllDebug("waitForCloseAllFinish(), catch");
      finished = "false";
   }

   if (finished == "false") {
      timer = window.setTimeout("waitForCloseAllFinish()", 100);
   }
   else {
      closeAllDebug("close all processing finished");
      CAForm.elements["HAFOperator.closeAllFinished"].value = "false";
   }
}

// ****************************************************************************
// applicationCloseAllFormSubmit
//
// Calls a series of functions to invoke the application close all functionality. 
// Sets the values of the hidden fields on the form within the hidden iframe 
// accordingly and submits the form.
// ****************************************************************************
function applicationCloseAllFormSubmit() {
   setForm();
   setHiddenFields();
   CAForm.elements["HAFOperator.closeAllMechanism"].value = "application_close_all";
   CAForm.elements["HAFOperator.closeAllCaller"].value = window.name;
   window.document.forms["HAFForm"].elements["HAFOperator.closeAllFinished"].value == "false";
   CAForm.submit();
}

// ****************************************************************************
// applicationCloseAll
//
// Initial function called by invoking close all from an application page. 
// ****************************************************************************
function applicationCloseAll(operation, event) {
   closeAllDebug("applicationCloseAll()");

	applicationCloseAllFormSubmit();
   waitForApplicationCloseAllFinish(operation, event);
}

// ****************************************************************************
// waitForApplicationCloseAllFinish
//
// Timer function used to ensure that the processing for application invoked
// close all has finished, before submitting the application page that close
// all was called from.
// ****************************************************************************
function waitForApplicationCloseAllFinish(operation, event) {
   closeAllDebug("waitForApplicationCloseAllFinish('" + operation + "', '" + event + "')");
   if (window.document.forms["HAFForm"].elements["HAFOperator.closeAllFinished"].value == "false") {
      closeAllDebug("doing setTimeout");
      timer = window.setTimeout("waitForApplicationCloseAllFinish('" + operation + "', '" + event + "')", 1);
   }
   else {
      closeAllDebug("doing ExecuteMenuAfterAppCloseAll");
      ExecuteMenuAfterAppCloseAll(operation, event);
   }
   
   //ensure closeAllFinished flag is reset to false. Required for correct updating of the open windows listbox
   window.document.forms["HAFForm"].elements["HAFOperator.closeAllFinished"].value = "false";
}

// ****************************************************************************
// signOff
//
// Initial function called when signing off. Sets the values of the hidden fields
// on the form within the hidden iframe accordingly and submits the form.
// ****************************************************************************
function signOff() {
   closeAllDebug("signOff()");

   setForm();
   setHiddenFields();

   CAForm.elements["HAFOperator.closeAllMechanism"].value = "sign_off";
   CAForm.submit();
}

// ****************************************************************************
// signOffContinue
//
// Function called when a user presses 'Continue' on the sign off page.
// ****************************************************************************
function signOffContinue() {
   closeAllOpenWindows();
   opener.close();
   document.forms["signOffForm"].submit();
}

// ****************************************************************************
// setFormElements
//
// Sets the hidden fields on the form within the hidden iframe.
// ****************************************************************************
function setHiddenFields() {
   closeAllDebug("setHiddenFields()");

   CAForm.dse_processorState.value = "initial";
   CAForm.dse_processorId.value = "";

   CAForm.dse_nextEventName.value = "hafCloseAll.start";
   CAForm.dse_operationName.value = "hafCloseAll";

   CAForm.elements["HAFOperator.openWinsList"].value = getOpenWinsList();
}

// ****************************************************************************
// getOpenWinsList
//
// ****************************************************************************
function getOpenWinsList() {
   closeAllDebug("getOpenWinsList()");

   openWinsArray = new Array();    //array containing the processor id of all the open windows
   openWins = new String();        //commar separated string of processor ids
   var i = 0;

   if (isDesktop) {
      //create the array of processor ids
      for(appWindow in window.applicationWindows) {
         i++;
         procId = window.applicationWindows[appWindow].winObj.document.HAFForm.dse_processorId.value;
         openWinsArray[i] = procId;
      }
   }
   else {
      //create the array of processor ids
      for(appWindow in opener.window.applicationWindows) {
         if (opener.window.applicationWindows[appWindow].winObj.document.forms[0].name != "signOffForm") {
            i++;
            procId = opener.window.applicationWindows[appWindow].winObj.document.HAFForm.dse_processorId.value;
            openWinsArray[i] = procId;
         }
      }
   }

   //create the string of processor ids
   openWins = openWinsArray.toString();

   //the toString() method inserts a commar at the beginning of the string so this needs to be removed
   openWins = openWins.substr(1);

   return openWins;
}

// ****************************************************************************
// closeWindows
//
// ****************************************************************************
function closeWindows(winId) {
   closeAllDebug("closeWindows()");
   CAForm = document.forms["closeAllForm"];
   var mechanism = CAForm.elements["HAFOperator.closeAllMechanism"].value;
   
   // if call complete was called close all unprotected windows
   if (mechanism == "application_close_all") { 
      if (parent.document.msgApplet) {
         try {
            parent.document.msgApplet.closeAll();
         }
         catch (e) {
            // Do nothing as the installed extensions are not present on the client
         }
      }
      closeUnprotectedWindows(winId);

      for(appWindow in parent.window.applicationWindows) {
         if (parent.window.applicationWindows[appWindow].winObj.name == winId) {
            parent.window.applicationWindows[appWindow].winObj.document.forms[0].elements["HAFOperator.closeAllFinished"].value = "true";
         }
      }
   }

   // if close all was called then we initially check to see if there are any protected applications open.
   if (mechanism == "close_all_check_only") {
      if (!protectedCheck(mechanism)) {
         // if there are no protected applications open then a message requiring an ok or cancel response is displayed to the user.
         var response = confirm("This action will close all Browser Main Menu functions still running. Any unsaved changes will be lost. Select OK to proceed or Cancel to return to the system.");
         if (response) {
            // If they continue with close all then the form within the hidden iframe is submitted again, this time to close processors.
            CAForm.dse_processorState.value = "initial";
            CAForm.dse_processorId.value = "";
            CAForm.dse_nextEventName.value = "hafCloseAll.start";
            CAForm.dse_operationName.value = "hafCloseAll";
            CAForm.elements["HAFOperator.closeAllMechanism"].value = "close_all";
            CAForm.submit();
         }
      }
      else {
         parent.window.focus();
         alert("There are applications open that cannot be closed automatically. Complete and/or close them separately and then try again.");
      }
   }

   if (mechanism == "close_all") {
      if (!protectedCheck(mechanism)) {
            closeAllOpenWindows();
      }
   }
}

// ****************************************************************************
// closeUnprotectedWindows
//
// ****************************************************************************
function closeUnprotectedWindows(winId) {
   closeAllDebug("closeUnprotectedWindows()");

   // split the protected processor list into an array of individual processor ids
   protectedProcs = document.forms["closeAllForm"].elements["HAFOperator.protectedProcessorIds"].value;
   idList = protectedProcs.split(",");

   // for each window check if it is protected.
   for (appWindow in parent.window.applicationWindows) {
      var protectedWindow = false;
      for (i=0; i < idList.length; i++) {
         if (parent.window.applicationWindows[appWindow].winObj.document.forms["HAFForm"].dse_processorId.value == idList[i]) {
            closeAllDebug("closeUnprotectedWindows() : protected " + appWindow);
            protectedWindow = true;
         }
      }

      // If the window is not protected then close it.
      if (!protectedWindow) {
         closeAllDebug("closeUnprotectedWindows() : closing " + appWindow);
         parent.window.applicationWindows[appWindow].winObj.askToClose();         
         delete parent.window.applicationWindows[appWindow];	         
      }
   }
	//make sure the open windows listbox is updated
   parent.window.updateListbox();   
}

// ****************************************************************************
// protectedCheck
//
// Checks for protected applications
// ****************************************************************************
function protectedCheck(mech) {
   var anyProtected = true;

   if (mech == "sign_off") {
      // If sign off has been called the close all form must be referenced differently
      if (window.frames['closeAllIframe'].document.forms["closeAllForm"].elements["HAFOperator.protectedProcessorIds"].value == "") {
         // If the protectedProcessorIds field in the close all form is an empty string then there are no protected apps open.
         anyProtected = false;
      }
   }
   else {
      if (document.forms["closeAllForm"].elements["HAFOperator.protectedProcessorIds"].value == "") {
         // If the protectedProcessorIds field in the close all form is an empty string then there are no protected apps open.
         anyProtected = false;
      }
   }

   return anyProtected;
}

// ****************************************************************************
// closeAllOpenWindows
//
// ****************************************************************************
function closeAllOpenWindows() {
   closeAllDebug("closeAllOpenWindows()");

   if (opener) {
      // If opener is not null then assume we are an application window
      win = opener.window;
   }
   else {
      // Not app window so must be hidden iframe on desktop
      win = parent.window;
   }

   for (appWindow in win.applicationWindows) {
      // do not close the sign off window
      if (win.applicationWindows[appWindow].winObj.document.forms[0].name != "signOffForm") {
         win.applicationWindows[appWindow].winObj.askToClose();
         delete win.applicationWindows[appWindow];
      }
   }
	//make sure the open windows listbox is updated
   win.updateListbox();
}

// ****************************************************************************
// askToClose
// Tries to close this (app) window, but only if no other app window is in the
// process of closing. In this case it set's a timer to re-invoke this function
// and try again.
// ****************************************************************************
function askToClose() {
   var isIE = (!document.all) ? 0 : 1;
   if (opener != null && !opener.closed) {
      if (opener.pageClosingInUse) {
         timer = window.setTimeout("askToClose()", 100);
      }
      else {
         opener.pageClosingInUse = true;

			//flag used to stop pageClosing() in windowhandler.js from updating the listbox when
			//close all has been invoked
         window.document.forms["HAFForm"].elements["HAFOperator.closeAllFinished"].value = "yes";

         window.close();
         if (isIE) {
             // Do this here for IE, IWB does it at end of pageClosing in windowhandler.js
             opener.pageClosingInUse = false;
         }
      }
   }
}

function closeAllDebug(msg) {
  //var now = new Date();
  //_console = window.open("","console","width=600,height=300,resizable,scrollbars");
  //_console.document.writeln(now + " " + now.getMilliseconds() + " - " + msg + "<br>");
}
