/*****************************************************************************
 *   Module : deskTop.js
 *   Description: This file contains the functionality necessary to manage
 *                and control the DeskTop application software.
 *
 *   Traceability to design : Represents the DeskTop class.
*****************************************************************************/

// Holds a unique name for each window opened.
var windowName = 0;

// Hold the maximum number of application windows.
var desktop_maxWindows = 9;

/*
 *  This function puts focus on the parent window, if it exists and is not closed.
 */
function gotoParent() {
   if (opener != null && !opener.closed) {
      opener.focus();
   }
}

/*
 *  This function returns the currently active key.
 */
function hdtGetActiveKey() {
   return hksGetActiveKey();
}


/*
 *  This function places the desktop into a state in which its functions
 *  may be called.
 *  This function should be called before any other function.
 */
function hdtInitialise() {
   hksInitialise();
   hdtShowMenu();
}


/*
 *  This function creates a key from the parameters passed to it
 *  before setting this key to the active key.
 */
function hdtAppWindowOpened(keyId, keyValue, keyDescription, keyType) {
   // Create a key object from component parts.
   var theKey = (keyId==null) ? null : (new key(keyId, keyValue, keyDescription, keyType));
   sendKeyToApplet(keyId,keyValue,keyDescription,keyType);
   hksSetActiveKey(theKey); 
}
/*
 *  This function is activated on a change of the drop down list and constructs
 *  a key from the constituent parts passed to it from the Application Window
 *  and set it to the active key.
 */
 function hdtAppWindowChangedKey(keyId, keyValue, keyDescription, keyType) {
    var theKey = (keyId==null) ? null : (new key(keyId, keyValue, keyDescription, keyType));
    sendKeyToApplet(keyId,keyValue,keyDescription,keyType);
    hksSetActiveKey(theKey);
 }

/*
 *  This function clears the active key before the child window is closed.
 */
function hdtAppWindowClosed() {
   hksSetActiveKey(null);
}


/*
 *  This function launches the required application along with
 *  the user selected parameters in a new window.
 *  If the user has exceeded the maximum amount of open windows
 *  the function will return false.
 */
function hdtLaunchApplication(name, requiredKey, windowOptions, processorName, applicationName) {

   // Holds the components required to build a key.
   var keyValue         = null;
   var keyId            = null;
   var keyDescription   = null;
   var keyType          = null;
   var SIGNOFF          = "signoff";

	//check the length of the applicationWindows array to see if the maximum number of windows are already open
   if (getApplicationWinsLength() >= desktop_maxWindows){
      // alert message signed off from business.
      alert("Maximum number of application windows already open.  Please close or complete one or more other open applications and try again.");
      return false;
   }
   
   // Extract the key component values from the form.

   // User selected KeyPanel data
   index = document.keyPanel.displayedKeyValue.selectedIndex;

   // Extract individual key components.
   selectValue_ = document.keyPanel.displayedKeyValue[index].value;
   selectText_  = document.keyPanel.displayedKeyValue[index].text;

   fieldSeparator = "::";
   selectValue = selectValue_.split(fieldSeparator);

   keyId    = selectValue[0];
   keyValue = selectValue[1];
   keyType  = selectValue[2];
   keyDescription  = selectValue[3];

   // Populate hidden fields with extracted data.
   for ( var i=0; i< document.DesktopForm.length; i++ ) {
      if ( document.DesktopForm.elements[i].type == "hidden") {
         switch ( document.DesktopForm.elements[i].name ) {
            case "HAFOperator.selectedKeyType" :
               document.DesktopForm.elements[i].value = keyId;
               break;
            case "HAFOperator.selectedKeyValue" :
              document.DesktopForm.elements[i].value = keyValue;
              break;
            case "HAFOperator.selectedKeyDescription" :
              document.DesktopForm.elements[i].value = keyDescription;
              break;
            case "HAFOperator.selectedKeySubType" :
              document.DesktopForm.elements[i].value = keyType;
              break;
            case "HAFOperator.selectedApplicationId" :
              document.DesktopForm.elements[i].value = name;
              break;
            case "HAFOperator.selectedApplicationKeyType" :
              document.DesktopForm.elements[i].value = requiredKey;
              break;      
            case "HAFOperator.processorName" :
              document.DesktopForm.elements[i].value = processorName;
              break;
            case "HAFOperator.applicationName" :
              document.DesktopForm.elements[i].value = applicationName;
              break;
         }
      }
   }


   // Values are fixed and determined by the framework setup.
   document.DesktopForm.dse_processorState.value = "initial";
   document.DesktopForm.dse_processorId.value = "";
   // signoff has a special processor
   if (name == SIGNOFF) {
      window.frames['closeAllIframe'].document.forms["closeAllForm"].elements["HAFOperator.closeAllFinished"].value = "false";
      signOff();
      waitForSignOffFinish(windowOptions)
      return;
   }
   else {
      document.DesktopForm.dse_nextEventName.value = "ApplicationLaunchProcessor.start";
      document.DesktopForm.dse_operationName.value = "ApplicationLaunchProcessor";
   }

   hdtLaunchAppSubmit(windowOptions);
}

function waitForSignOffFinish(windowOptions) {
   try {
      finished = window.frames['closeAllIframe'].document.forms["closeAllForm"].elements["HAFOperator.closeAllFinished"].value;
   }
   catch (ex) {
      finished = "false";
   }

   if (finished == "false") {
      timer = window.setTimeout("waitForSignOffFinish('" + windowOptions + "')", 1);
   }
   else {
      //alert("wait");
      //alert(window.frames['closeAllIframe'].document.forms["closeAllForm"].elements["HAFOperator.protectedProcessorIds"].value);
      var mechanism = window.frames['closeAllIframe'].document.forms["closeAllForm"].elements["HAFOperator.closeAllMechanism"].value;
      if (!protectedCheck(mechanism)) {
         document.DesktopForm.dse_nextEventName.value = "hafSignOffProc.start";
         document.DesktopForm.dse_operationName.value = "hafSignOffProc";
         hdtLaunchAppSubmit(windowOptions);
      }
      else {
         //alert("some protected");
         alert("There are applications open that cannot be closed automatically. Complete and/or close them separately and then try again.");
      }
      window.frames['closeAllIframe'].document.forms["closeAllForm"].elements["HAFOperator.closeAllFinished"].value = "false";
   }
}

function hdtLaunchAppSubmit(windowOptions) {
   var CARM_PREFIX      = "CRM";
   var BLANK_URL        = "ABOUT:BLANK";

   // Launch application in a unique window (unless it is CARM).
   if (CARM_PREFIX == name.substr(0, CARM_PREFIX.length).toUpperCase()) {
      // Open or get reference to CARM window and bring it to front
      carmWindow = window.open('', CARM_PREFIX, windowOptions);
      carmWindow.focus();

      if (BLANK_URL == carmWindow.location.toString().toUpperCase()) {
         // Window location is empty (i.e. it's just opened) so set and load appropriate target
         document.DesktopForm.target = CARM_PREFIX;
         document.forms["DesktopForm"].submit();
      }
   }
   else {
      windowName++;
      newWindow = window.open('', windowName, windowOptions);

      newWindow.focus();
      document.DesktopForm.target = windowName;
      document.forms["DesktopForm"].submit();
   }

}

function getKeysServicesKeys() {
   return document.keyPanel.displayedKeyValue;
}

/* finals for params for getKeyTypesForApp */
var KLC_KEYS = 1;
var PSL_KEYS = 2;

function getKeyTypesForApp(reqKeyTypes) {
   var allKeys = document.keyPanel.displayedKeyValue;
   var appKeys = new Array(0);
   var appKeysCount = 0;

   if (allKeys != null && allKeys.options.length > 0)
   {
      for (var i = 0; i < allKeys.options.length; i++)
      {
         var s = allKeys.options[i].value;
         var subString = s.substring(0,1);

         if (reqKeyTypes == KLC_KEYS) {
            if (subString == Card || subString == Product || subString == Customer) {
               appKeys[appKeysCount] = allKeys.options[i];
               appKeysCount++;
            }
         }
         else if (reqKeyTypes == PSL_KEYS) {
            if(subString == Staff || subString == Portfolio){
               appKeys[appKeysCount] = allKeys.options[i];
               appKeysCount++;
            }
         }
      }
   }

   if (appKeys.length > 0) {
      return appKeys;
   }
   else {
      return null;
   }
}

function getKLCKeys() {
   return getKeyTypesForApp(KLC_KEYS);
}

function getPSLKeys() {
   return getKeyTypesForApp(PSL_KEYS);
}

/* Displays the technical details of this session on the desktop*/
function launchTechDets() {
     var tableId = document.getElementById("TechDet");
       if (tableId.style.visibility=="visible"){
            hideTechDets();
            }
       else{
            tableId.style.visibility="visible";
            tableId.focus();
       }

}

/* Launch new window to display the technical details of this session */
function hideTechDets() {
     var tableId = document.getElementById("TechDet");
       tableId.style.visibility="hidden";

}

// Passes the key information to the XML Messaging Applet
function sendKeyToApplet(keyId,keyValue,keyDescription,keyType) {
   // Checks for null value 
   if(keyId != null && keyId != -1) {
      // Passes the key details to the XML Messaging Applet
      if(document.msgApplet != null) {
         try {
            document.msgApplet.sendKey(keyType, keyValue, keyDescription);
         }
         catch(e) {
            // Do nothing as the installed extensions are not present on the client
         }
      }
   }
}
