// Logging Constants
var LOG_NONE      = 0;
var LOG_EVENT     = 1;
var LOG_FUNC      = 2;
var LOG_UTIL_FUNC = 4;
var LOG_DEBUG     = 8;
var LOG_HTML      = 16;
var LOG_ALL       = 255;

var logLevel = LOG_NONE;
var __j = location.href.indexOf("logLevel=");
if (__j > -1) {

   var s = location.href.substring(__j+9);
   var value = parseInt(s);
   if (!isNaN(value)) {
      logLevel = value;
   }
}

// Misc. Constants
var SHOWN            = "block";
var HIDDEN           = "";
var ID_ITEM          = "menuItem_";
var ID_CONTAINER     = "menuContainer_";
var ID_TABLE_ROW     = "tableRow_";
var MENU_TYPE        = "1";
var ITEM_TYPE        = "2";
var SUB_TYPE         = "3";
var HIGHLIGHT_STYLE  = "hMenubarH";
var NORMAL_STYLE     = "hMenubar";
var TABBABLE         = new Array("A", "AREA", "BUTTON", "INPUT", "OBJECT", "SELECT", "TEXTAREA");

// Hotkeys
var A_TO_Z           = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
var ZERO_TO_NINE     = "0123456789";

// Key Codes
var KEY_A            = 65;
var KEY_0            = 48;
var KEY_NUMPAD_0     = 96;
var KEY_NUMPAD_9     = 105;
var KEY_TAB          = 9;
var KEY_ESCAPE       = 27;
var KEY_LEFT         = 37;
var KEY_UP           = 38;
var KEY_RIGHT        = 39;
var KEY_DOWN         = 40;

// Global Variables
// Stores the number of level 3 menu items belonging to each level 2 menu
// e.g. the number of items in the third item of the first menu would be == numSubsArray[0][2]
// 0 for level 2 items with no level 3 items
// e.g the number of menu items in the 3rd menu would be numSubsArray[2].length
var numSubsArray     = new Array();

// Saves which menu is currently dropped (object type MenuNumber)
var menuDropped      = new MenuNumber(null, null, null);

// Saves which menu item is currently highlighted (object type MenuNumber)
var highlighted      = new MenuNumber(null, null, null);

// Operating System we are running on
var thisOS           = navigator.platform.toLowerCase();
var os2              = "warp";

// Current log line number
var logline          = 1;

// Graphics, actual values are set by the menu bean
var imgRight         = "";
var imgDown          = "";

// Used to track duplicate events raised by IWB - specifically cursor key right and menu hot keys
// which cause the level 3 menu to be drawn twice giving a flashing appearance. Can be removed
// when (if?) this bug is fixed in IWB.
var duplicateKeyEvent = false;

// *****************************************************************************
// Function:      Menu
// Parameter:     text     The menu text
//                type     The menu type (1, 2 or 3)
//                hotkey   The hotkey for the menu
//                op       The operation the menu should invoke
//                evt      The event the menu should invoke
// Returns:       Nothing
// Description:   Menu Constructor
// *****************************************************************************
function Menu(text, type, hotkey, op, evt, appId, keyType, closeAllReq, processorName, applicationName) {
   this.text = text;
   this.type = type;
   hotkey = hotkey.toUpperCase();
   if ((A_TO_Z.indexOf(hotkey) < 0) && (ZERO_TO_NINE.indexOf(hotkey) < 0)) {
      alert("Unacceptable hotkey=" + hotkey);
   }

   this.hotkey = hotkey;
   this.op = op;
   this.evt = evt;
   this.appId = appId;
   this.keyType = keyType;
   this.items = null;
   this.closeAllReq = closeAllReq;
   this.processorName = processorName;
   this.applicationName = applicationName;
}

// *****************************************************************************
// Function:      IncompatibleBrowser
// Parameter:     None
// Returns:       true if compatible, else false
// Description:   Check that the browser is compatible with this JavaScript
// *****************************************************************************
function IncompatibleBrowser() {
   // Are we using block?
   if (navigator.userAgent.toLowerCase().indexOf("gecko") == -1) {
      alert("This HMenu JavaScript will not run with this browser version.\n"+navigator.userAgent.toLowerCase());
      return true;
   }

   return false;
}

// *****************************************************************************
// Function:      MenuNumber
// Parameter:     m  level 1 menu id
//                i  level 2 menu id
//                s  level 3 menu id
// Returns:       Nothing
// Description:   MenuNumber Constructor
// *****************************************************************************
function MenuNumber(m, i, s) {
   this.m = m;
   this.i = i;
   this.s = s;
   this.toString = MenuNumber_toString;
}

// *****************************************************************************
// Function:      MenuNumber
// Parameter:     None
// Returns:       A menu number object as a string for display
// Description:
// *****************************************************************************
function MenuNumber_toString() {
   return "[m=" + this.m + " i=" + this.i + " s=" + this.s + "]";
}

// *****************************************************************************
// Function:      _Log
// Parameter:     s  Text to be logged
// Returns:       Nothing
// Description:
// *****************************************************************************
function _Log(s) {
   s = s.replace(/</g, "&lt")
   s = s.replace(/>/g, "&gt")
   domDebug(logline + "  " + s);
   logline++;
}

// *****************************************************************************
// Function:      Log
// Parameter:     s  Text to be logged
// Returns:       Nothing
// Description:
// *****************************************************************************
function Log(s) {
   if ((logLevel & LOG_DEBUG) > 0) {
      _Log("        " + s);
   }
}

// *****************************************************************************
// Function:      LogFunc
// Parameter:     f  Function name doing the logging
//                s  Text to be logged
// Returns:       Nothing
// Description:
// *****************************************************************************
function LogFunc(f, s) {
   if ((logLevel & LOG_FUNC) > 0) {
      _Log(f + ": " + s);
   }
}

// *****************************************************************************
// Function:      LogUtilFunc
// Parameter:     f  Function name doing the logging
//                s  Text to be logged
// Returns:       Nothing
// Description:
// *****************************************************************************
function LogUtilFunc(f, s) {
   if ((logLevel & LOG_UTIL_FUNC) > 0) {
      _Log(f + ": " + s);
   }
}

// *****************************************************************************
// Function:      LogEvent
// Parameter:     e  Event to be logged
//                s  Text to be logged
// Returns:       Nothing
// Description:
// *****************************************************************************
function LogEvent(e, s) {
   if ((logLevel & LOG_EVENT) > 0) {
      _Log(e + ": " + s);
   }
}

// *****************************************************************************
// Function:      WriteHTML
// Parameter:     s  HTML to be written
// Returns:       Nothing
// Description:
// *****************************************************************************
function WriteHTML(s) {
   document.writeln(s);
   if ((logLevel & LOG_HTML) > 0) {
      _Log(s);
   }
}

// *****************************************************************************
// Function:      UnderlineChar
// Parameter:     text        Text to have underlined character inserted
//                character   Character to underline
// Returns:       Supplied text with an underlined character (if it was found).
// Description:   Returns the HTML for a menu with an underlined character
// *****************************************************************************
function UnderlineChar(text, character) {
   if (0 == character.length) {
      // Nothing to do (empty string)
      return text;
   }

   var uText = text.toUpperCase();
   var uC = character.toUpperCase();
   if (-1 == uText.indexOf(uC)) {
      // Nothing to do (character not found in text)
      return text;
   }

   newText = text.substr(0, uText.indexOf(uC));
   newText += "<span class=\"hMenuUnderline\">";
   newText += text.substr(uText.indexOf(uC), 1);
   newText += "</span>";
   newText += text.substr(uText.indexOf(uC) + 1);
   return newText;
}

// *****************************************************************************
// Function:      EndSub
// Parameter:     index Number of level 3 menu items
//                m     Level 1 menu id
//                i     Level 2 menu id
//                s     Level 3 menu id
// Returns:       Nothing
// Description:
// *****************************************************************************
function EndSub(index, m, i, s) {
   if (!numSubsArray[m]) {
      numSubsArray[m] = new Array();
   }
   numSubsArray[m][i] = s+1;

   if (-1 != s) {
      WriteHTML("</table></div>");
      // add all the sub items to the parent item's items list
      var start = index-(1+(s+1));
      var parent = menu[start];
      parent.items = new Array();
      for (var x = start+1; x < index; x++) {
         Log("Adding " + menu[x].text + " to " + parent.text);
         parent.items[parent.items.length] = menu[x];
      }
   }
}

// *****************************************************************************
// Function:      WriteSubs
// Parameter:     None
// Returns:       Nothing
// Description:   Writes the DIVs for the sub menu drop downs
//                This function writes one DIV per sub menu
//                Each sub menu DIV consists of a table, and the sub menu items are
//                one per table row.
//                The numSubsArray variable is updated with the number of sub menu items per
//                menu item, 0 if there are none.
// *****************************************************************************
function WriteSubs() {
   var m = -1;
   var i = -1;
   var s  = -1;

   var x = 0;
   while (x < menu.length) {
      if (MENU_TYPE == menu[x].type) {
         EndSub(x, m, i, s);
         m++;
         i = -1;
         s = -1;
      }
      else if (ITEM_TYPE == menu[x].type) {
         EndSub(x, m, i, s);
         i++;
         s = -1;
      }
      else if (SUB_TYPE == menu[x].type) {
         // if this is the first sub item, start the div and the table
         if (-1 == s) {
            WriteHTML("<div class=\"hMenuContainer\" id=\"" + ID_CONTAINER + m + "_" + i + "\">" +
              "<table border=0 cellpadding=2 cellspacing=0 width=\"100%\">");
         }
         s++;

         WriteHTML("<tr id=\"tableRow_" + m + "_" + i + "_" + s + "\"" +
           " class=\"hMenubar\" width=\"100%\"" +
           " onclick=\"MouseClick(event, " + m + "," + i + "," + s + ");");
         if (((null != menu[x].op) && (menu[x].op.length > 0))
          || ((null != menu[x].evt) && (menu[x].evt.length > 0))
          || ((null != menu[x].appId) && (menu[x].appId.length > 0))) {
            WriteHTML(" ExecuteMenu(" + m + "," + i + "," + s + ");");
         }
         WriteHTML("\">");

         var href = "javascript:NullEvent()";

         WriteHTML("<td>" +
           "<a class=\"hMenubar\"" +
           " id=\"" + ID_ITEM + m + "_" + i + "_" + s + "\"" +
           " onblur=\"MenuBlur(" + m + "," + i + "," + s + ")\"" +
           " href=\"" + href + "\""
           + getOnClickHandlerFor(href)
           + ">" +
           UnderlineChar(menu[x].text, menu[x].hotkey) +
           "</a></td></tr>");
      }
      x++;
   }

   EndSub(x, m, i, s);
}

// *****************************************************************************
// Function:      EndItem
// Parameter:     index Number of level 2 menu items
//                m     Level 1 menu id
//                i     Level 2 menu id
// Returns:       Nothing
// Description:
// *****************************************************************************
function EndItem(index, i, s) {
   if (-1 != i) {
      WriteHTML("</table></div>");
      // add all the items to the parent menu's items list
      var start = index-(1+(i+1)+(s+1));
      var parent = menu[start];
      parent.items = new Array();
      for (var x = start+1; x < index; x++) {
         if (ITEM_TYPE == menu[x].type) {
            Log("Adding " + menu[x].text + " to " + parent.text);
            parent.items[parent.items.length] = menu[x];
         }
      }
   }
}

// *****************************************************************************
// Function:      WriteItems
// Parameter:     None
// Returns:       Nothing
// Description:   Writes the DIVs for the menu drop downs
//                Each menu DIV consists of a table and one menu item per table row.
//                If the menu item has sub items, a second table cell is added to the row
//                containing a right arrow.
// *****************************************************************************
function WriteItems() {
   var m = -1;
   var i = -1;
   var s = -1;

   var x = 0;
   while (x < menu.length) {
      if (MENU_TYPE == menu[x].type) {
         EndItem(x, i, s);

         m++;
         i = -1;
         s = -1;
      }
      else if (ITEM_TYPE == menu[x].type) {
         if (-1 == i) {
            WriteHTML("<div class=\"hMenuContainer\" id=\"" + ID_CONTAINER + m + "\">" +
              "<table border=0 cellpadding=2 cellspacing=0 width=\"100%\">");
         }
         i++;

         var href = "javascript:NullEvent()";
         if (menu[x].appId == "signoff") {
            WriteHTML("<tr id=\"tableRow_" + m + "_" + i + "\"" +
              " class=\"hMenubar\" width=\"100%\"" +
              " onclick=\"MouseClick(event, " + m + "," + i + ")\">");

            if (((null != menu[x].op) && (menu[x].op.length > 0))
             || ((null != menu[x].evt) && (menu[x].evt.length > 0))
             || ((null != menu[x].appId) && (menu[x].appId.length > 0))) {
               href = "javascript:ExecuteMenu(" + m + "," + i + ")";
            }
         }
         else {
            WriteHTML("<tr id=\"tableRow_" + m + "_" + i + "\"" +
              " class=\"hMenubar\" width=\"100%\"" +
              " onclick=\"MouseClick(event, " + m + "," + i + ");");

            if (((null != menu[x].op) && (menu[x].op.length > 0))
             || ((null != menu[x].evt) && (menu[x].evt.length > 0))
             || ((null != menu[x].appId) && (menu[x].appId.length > 0))) {
               WriteHTML(" ExecuteMenu(" + m + "," + i + ");");
            }

            WriteHTML("\">");

            href = "javascript:NullEvent()";
         }

         var a = "<a class=\"hMenubar\"" +
           " id=\"" + ID_ITEM + m + "_" + i + "\"" +
           " onblur=\"MenuBlur(" + m + "," + i + ")\"" +
           " onfocus=\"MenuFocus(" + m + "," + i + ")\"" +
           " href=\"" + href + "\""
           + getOnClickHandlerFor(href)
           + ">" + UnderlineChar(menu[x].text, menu[x].hotkey) + "</a>";

         if (HasItems(m, i)) {
            WriteHTML("<td width=\"100%\">" +
              a + "<td width=10>" +
              "<img class=\"hMenubar\" id=\"img_" + m + "_" + i + "\" " + imgRight + ">");
         }
         else {
            WriteHTML("<td colspan=2>" + a);
         }

         WriteHTML("</td></tr>");
      }
      else if (SUB_TYPE == menu[x].type) {
         s++;
      }
      x++;
   }

   EndItem(x, i, s);
}

// *****************************************************************************
// Function:      WriteMenu
// Parameter:     None
// Returns:       Nothing
// Description:   Writes the menu
//                Constructs a table with the top level menu items. If a menu
//                has drop down items a down arrow is drawn next to the menu
//                text.
// *****************************************************************************
function WriteMenu() {
   if (IncompatibleBrowser()) {
      return;
   }

   if (logLevel > 0) {
      document.writeln("<form><textarea id=\"logBox\" style=\"font-size:8pt;\" wrap=\"off\" cols=\"76\" rows=\"18\"></textarea></form>");
      document.getElementById("logBox").value = "";
   }

   WriteSubs();
   WriteItems();
   WriteHTML("<table border=0 cellpadding=2 cellspacing=0 width=\"100%\"><tr class=\"hMenubar\">");
   var m = -1;
   for (var x = 0; x < menu.length; x++) {
      if (MENU_TYPE == menu[x].type) {
         m++;

         var img = "";
         if (HasItems(m)) {
            img = "<img class=\"hMenubar\" " + imgDown + ">";
         }

         WriteHTML("<td nowrap width=\"0%\" class=\"hMenubar\" id=\"tableCell_" + m + "\"" +
           " onclick=\"MouseClick(event, " + m + ");");
            if (((null != menu[x].op) && (menu[x].op.length > 0))
             || ((null != menu[x].evt) && (menu[x].evt.length > 0))
             || ((null != menu[x].appId) && (menu[x].appId.length > 0))) {
               WriteHTML(" ExecuteMenu(" + m + ");");
            }
         WriteHTML("\">");

         var href = "javascript:NullEvent()";

         WriteHTML("<a class=\"hMenubar\"" +
            " id=\"" + ID_ITEM + m + "\"" +
            " onblur=\"MenuBlur(" + m + ")\"" +
            " onfocus=\"MenuFocus(" + m + ")\"" +
            " accesskey=\"" + menu[x].hotkey + "\"" +
            " href=\"" + href + "\""
           + getOnClickHandlerFor(href)
            + ">" + UnderlineChar(menu[x].text, menu[x].hotkey) + img + "</a>&nbsp;&nbsp;" +
            "</td>");
      }
   }
   WriteHTML("<td width=\"100%\" id=\"endOfMenu\">&nbsp;</td></tr></table>");

   window.addEventListener("mousedown", MouseDown, false);
   window.addEventListener("keyup", KeyUp, false);
}

// *****************************************************************************
// Function:      GetMenu
// Parameter:     m  level 1 menu id
//                i  level 2 menu id
//                s  level 3 menu id
// Returns:       The menu object associated with the parameters
// Description:
// *****************************************************************************
function GetMenu(m, i, s) {

   var index = 0;

   // add up all the entries before the menu we want (m)
   for (var x = 0; x < m; x++) {
      // add one for the top level menu item
      index++;

      // add the number of items in the menu
      index += numSubsArray[x].length;

      // add all the sub items in the menu
      for (var y = 0; y < numSubsArray[x].length; y++) {
         index += numSubsArray[x][y];
      }
   }

   var result = menu[index];
   if (null == result) {
      return null;
   }

   // add up all the menu items of the menu we want up to the actual menu item we want (i)
   if ((null != i) && (!isNaN(i))) {
      result = result.items[i];
   }

   if ((null != s) && (!isNaN(s))) {
      result = result.items[s];
   }

   return result;
}

// *****************************************************************************
// Function:      GetTableRow
// Parameter:     m  level 1 menu id
//                i  level 2 menu id
//                s  level 3 menu id
// Returns:       A table row element matching the supplied parameters
// Description:   Gets a table row
// *****************************************************************************
function GetTableRow(m, i, s) {
   LogUtilFunc("GetTableRow", "");
   var name = "tableRow_" + m;

   if ((null != i) && (!isNaN(i))) {
      name += "_" + i;
   }

   if ((null != s) && (!isNaN(s))) {
      name += "_" + s;
   }

   return document.getElementById(name);
}

// *****************************************************************************
// Function:      GetTableCell
// Parameter:     m  level 1 menu id
//                i  level 2 menu id
//                s  level 3 menu id
// Returns:       A table cell element matching the supplied parameters
// Description:   Gets a table cell
// *****************************************************************************
function GetTableCell(m, i, s) {
   LogUtilFunc("GetTableCell", "");
   var name = "tableCell_" + m;

   if ((null != i) && (!isNaN(i))) {
      name += "_" + i;
   }

   if ((null != s) && (!isNaN(s))) {
      name += "_" + s;
   }

   return document.getElementById(name);
}

// *****************************************************************************
// Function:      IsTableRow
// Parameter:     element  The element to be checked
// Description:
// *****************************************************************************
function IsTableRow(element) {
   LogUtilFunc("IsTableRow", "");
   if ((null == element) || (null == element.id)) {
      return false;
   }
   return ("tableRow_" == element.id.substring(0, 9));
}

// *****************************************************************************
// Function:      GetMenuAnchor
// Parameter:     m  level 1 menu id
//                i  level 2 menu id
//                s  level 3 menu id
// Returns:       An anchor element matching the supplied parameters
// Description:   Returns the menu anchor with the specified number
// *****************************************************************************
function GetMenuAnchor(m, i, s) {
   LogUtilFunc("GetMenuAnchor", "m=" + m + ", i=" + i + ", s=" + s);
   var name = ID_ITEM + m;

   if ((null != i) && (!isNaN(i))) {
      name += "_" + i;
   }

   if ((null != s) && (!isNaN(s))) {
      name += "_" + s;
   }

   return document.getElementById(name);
}

// *****************************************************************************
// Function:      GetNextMenuAnchor
// Parameter:     m  level 1 menu id
//                i  level 2 menu id
//                s  level 3 menu id
// Returns:       See description
// Description:   Returns the 'next' menu anchor. If the parameters represent a
//                sub menu item, this function will return the next sub menu item. If
//                the parameters represent a menu item, this function will return the
//                next menu item. If the parameters represent a top level menu, this
//                function will return the next top level menu.
// *****************************************************************************
function GetNextMenuAnchor(m, i, s) {
   LogUtilFunc("GetNextMenuAnchor", "m=" + m + ", i=" + i + ", s=" + s);
   if ((null != s) && (!isNaN(s))) {
      s++;
      if (s >= numSubsArray[m][i]) {
         s = 0;
      }
   }
   else if ((null != i) && (!isNaN(i))) {
      i++;
      if (i >= numSubsArray[m].length) {
         i = 0;
      }
   }
   else {
      m++;
      if (m >= numSubsArray.length) {
         m = 0;
      }
   }
   return GetMenuAnchor(m, i, s);
}

// *****************************************************************************
// Function:      GetPrevMenuAnchor
// Parameter:     m  level 1 menu id
//                i  level 2 menu id
//                s  level 3 menu id
// Returns:       See description
// Description:   Returns the 'prev' menu anchor. If the parameters represent a
//                sub menu item, this function will return the prev sub menu item. If
//                the parameters represent a menu item, this function will return the
//                prev menu item. If the parameters represent a top level menu, this
//                function will return the prev top level menu.
// *****************************************************************************
function GetPrevMenuAnchor(m, i, s) {
   LogUtilFunc("GetPrevMenuAnchor", "m=" + m + ", i=" + i + ", s=" + s);
   if ((null != s) && (!isNaN(s))) {
      s--;
      if (s < 0) {
         s = numSubsArray[m][i]-1;
      }
   }
   else if ((null != i) && (!isNaN(i))) {
      i--;
      if (i < 0) {
         i = numSubsArray[m].length-1;
      }
   }
   else {
      m--;
      if (m < 0) {
         m = numSubsArray.length-1;
      }
   }
   return GetMenuAnchor(m, i, s);
}

// *****************************************************************************
// Function:      GetContainer
// Parameter:     m  level 1 menu id
//                i  level 2 menu id
// Returns:       A container element matching the supplied parameters
// Description:   Returns the container (DIV) specified by the parameters.
//                If parmeter i is passed in, we return a sub item container,
//                else we return a menu container.
// *****************************************************************************
function GetContainer(m, i) {
   LogUtilFunc("GetContainer", "m=" + m + " i=" + i);
   var name = ID_CONTAINER + m;

   if ((null != i) && (!isNaN(i))) {
      name += "_" + i;
   }

   //Log("name=" + name);
   return document.getElementById(name);
}

// *****************************************************************************
// Function:      HasItems
// Parameter:     m  level 1 menu id
//                i  level 2 menu id
// Returns:       Returns true if a menu has items.
// Description:
// *****************************************************************************
function HasItems(m, i) {
   LogUtilFunc("HasItems", "");
   var a = numSubsArray[m];

   if ((null == a) || (a.length == 0)) {
      return false;
   }

   if ((null != i) && (!isNaN(i))) {
      var a = a[i];
      return ((null != a) && (a > 0));
   }

   return ((null != a) && (a.length > 0));
}

// *****************************************************************************
// Function:      IsHiddenC
// Parameter:     container   Container element to be checked
// Returns:       Returns true if a container is hidden.
// Description:
// *****************************************************************************
function IsHiddenC(container) {
   LogUtilFunc("IsHiddenC", "");
   return (HIDDEN == container.style.display);
}

// *****************************************************************************
// Function:      IsMenuAnchor
// Parameter:     Anchor element to be checked
// Returns:       Returns true if the element is a menu anchor.
// Description:
// *****************************************************************************
function IsMenuAnchor(element) {
   LogUtilFunc("IsMenuAnchor", "");
   if ((null == element) || (!element.id)) {
      return false;
   }
   return (ID_ITEM == element.id.substring(0, 9));
}

// *****************************************************************************
// Function:      HideAll
// Parameter:     None
// Returns:       Nothing
// Description:   Hides all the menus.
// *****************************************************************************
function HideAll() {
   LogUtilFunc("HideAll", "");
   for (var m = 0; m < numSubsArray.length; m++) {
      if (HasItems(m)) {
         HideMenu(m);
      }
      for (var i = 0; i < numSubsArray[m].length; i++) {
         if (HasItems(m, i)) {
            HideSub(m, i);
         }
      }
   }
   LoseHighlight(highlighted);
}

// *****************************************************************************
// Function:      HideC
// Parameter:     Container element to be hidden
// Returns:       Nothing
// Description:   Hides a specific container.
// *****************************************************************************
function HideC(container) {
   LogUtilFunc("HideC", "container=" + container.id);
   if (null != container) {
      container.style.display = HIDDEN;
   }
}

// *****************************************************************************
// Function:      HideMenu
// Parameter:     m  level 1 menu id
// Returns:       Nothing
// Description:   Hides the m'th menu. Zero based index.
// *****************************************************************************
function HideMenu(m) {
   LogUtilFunc("HideMenu", "m=" + m);
   if (null == m) {
      return;
   }

   var container = GetContainer(m);
   if (null != container) {
      HideC(container);
   }
}

// *****************************************************************************
// Function:      HideSub
// Parameter:     m  level 1 menu id
//                i  level 2 menu id
// Returns:       Nothing
// Description:   Hides the sub menu of the i'th menu item of the m'th menu. Zero based index.
// *****************************************************************************
function HideSub(m, i) {
   LogUtilFunc("HideSub", "m=" + m + ", i=" + i);
   if ((null == m) || (null == i)) {
      return;
   }

   var container = GetContainer(m, i);
   if (null != container) {
      HideC(container);
   }
}

// *****************************************************************************
// Function:      ShowC
// Parameter:     Container element to be shown
// Returns:       Nothing
// Description:
// *****************************************************************************
function ShowC(container) {
   LogUtilFunc("ShowC", "container=" + container.id);
   if (null != container) {
      PutMenuInRightPos(container);
      container.style.display = SHOWN;
   }
}

// *****************************************************************************
// Function:      ShowMenu
// Parameter:     m  level 1 menu id
// Returns:       Nothing
// Description:
// *****************************************************************************
function ShowMenu(m) {
   LogUtilFunc("ShowMenu", "m=" + m);
   var container = GetContainer(m);
   if (null != container) {
      ShowC(container);
      menuDropped.m = m;
      menuDropped.i = null;
   }
}

// *****************************************************************************
// Function:      ShowSub
// Parameter:     m  level 1 menu id
//                i  level 2 menu id
// Returns:       Nothing
// Description:
// *****************************************************************************
function ShowSub(m, i) {
   LogUtilFunc("ShowSub", "m=" + m + " i=" + i);
   var container = GetContainer(m, i);
   if (null != container) {
      ShowC(container);
      menuDropped.m = m;
      menuDropped.i = i;
   }
}

// *****************************************************************************
// Function:      GetPageSize
// Parameter:     None
// Returns:       A Point object containing the size of the document window.
// Description:
// *****************************************************************************
function GetPageSize() {
   LogUtilFunc("GetPageSize", "");
   var p = new Point(0, 0);
   p.x = window.innerWidth-8;
   p.y = window.innerHeight-8;
   return p;
}

// *****************************************************************************
// Function:      PutMenuInRightPos
// Parameter:     Container element to be positioned
// Returns:       Nothing
// Description:   Puts the menu container in the correct position.
// *****************************************************************************
function PutMenuInRightPos(container) {
   LogUtilFunc("PutMenuInRightPos", "container=" + container);

   // Cannot use extract menu number because this is a container and not a menu
   var menuNumber = ExtractMenuNumber(container);
   var m = menuNumber.m;
   var i = menuNumber.i;
   var menuType = GetMenuType(m, i);
   var pageSize = GetPageSize();

   // For some reason we need to have the DIV visible to know its size
   // so flip it on and off quick
   container.style.display = SHOWN;

   var w = container.offsetWidth;
   var h = container.offsetHeight;
   var tables = container.getElementsByTagName("TABLE");
   if ((null != tables) && (tables.length > 0)) {
      var table = tables[0];
      if (table.offsetWidth+10 > w) {
         w = table.offsetWidth+10;
         container.style.width = w;
      }
   }

   container.style.display = HIDDEN;
   // stores the eventual location of the menu
   var pos = null;

   if (MENU_TYPE == menuType) {
      // anchor a top level menu to its link
      var anchor = GetMenuAnchor(m);
      pos = GetMenuPos(anchor, false);

      pos.x += document.body.offsetLeft;
      pos.y += document.body.offsetTop;

      container.style.zIndex = 1000;

      // add the height of the link, so the menu appears below
      pos.y += anchor.offsetHeight;

      // if the menu would go off the right hand side of the screen, snap it to right side
      if ((pos.x + w) > pageSize.x) {
         Log(pos.x + "+" + w + " is too big for " + pageSize.x);
         pos.x = pageSize.x - w;
      }

      // if the menu would go off the left hand side of the screen, snap it to left side
      if (pos.x < 0) {
         pos.x = 0;
      }
   }
   else if (ITEM_TYPE == menuType) {
      // anchor a sub menu to its parent item's row
      var anchor = GetTableRow(m, i);
      pos = GetMenuPos(anchor, true);
      container.style.zIndex = 1001;

      // if the sub menu would go off the right hand side of the screen ...
      if ((pos.x + anchor.offsetWidth + w) > pageSize.x) {
         Log(pos.x + "+" + w + " is too big for " + pageSize.x);

         // if there is more space for the menu to the right, snap to right side
         if (pos.x > (pageSize.x - (pos.x + anchor.offsetWidth))) {
            pos.x = pos.x - w;
         }
         // else if there is more space for the menu to the left, snap to left side
         else {
            pos.x = pageSize.x - w;
         }

         // if the menu would go off the left hand side of the screen, snap it to left side
         if (pos.x < 0) {
            pos.x = 0;
         }
      }
      else {
         // enough room, display as normal
         pos.x += anchor.offsetWidth;
      }
   }

   container.style.left  = pos.x;
   container.style.top = pos.y;
}

// *****************************************************************************
// Function:      _Highlight
// Parameter:     ob Object to be highlighted
//                m  level 1 menu id
//                i  level 2 menu id
//                s  level 3 menu id
// Returns:       Nothing
// Description:
// *****************************************************************************
function _Highlight(ob, m, i, s) {
   LogUtilFunc("_Highlight", "");
   if (ob) {
      ob.className = HIGHLIGHT_STYLE;
      var a = GetMenuAnchor(m, i, s);
      if (null != a) {
         a.className = HIGHLIGHT_STYLE;
      }
   }
}

// *****************************************************************************
// Function:      Highlight
// Parameter:     ?
// Returns:       Nothing
// Description:
// *****************************************************************************
function Highlight(menuAnchor) {
   LogFunc("Highlight", "menuAnchor=" + menuAnchor + " menuAnchor.id=" + (menuAnchor != null ? menuAnchor.id : ""));
   var menuNumber = ExtractMenuNumber(menuAnchor);
   var m = menuNumber.m;
   var i = menuNumber.i;
   var s = menuNumber.s;
   var cell = GetTableCell(m);
   _Highlight(cell, m);
   highlighted.m = m;
   highlighted.i = i;
   highlighted.s = s;

   // Highlight the item and sub item
   if (MENU_TYPE != GetMenuType(m, i, s)) {
      if (null != i) {
         var row = GetTableRow(m, i);
         _Highlight(row, m, i);
         Log("H"+(row ? row.tagName : "null") + "-" + m + "," + i + "," + s);
      }

      if (null != s) {
         var row = GetTableRow(m, i, s);
         _Highlight(row, m, i, s);
         Log("H"+(row ? row.tagName : "null") + "-" + m + "," + i + "," + s);
      }
   }
}

// *****************************************************************************
// Function:      _LoseHighlight
// Parameter:     ob Object to be highlighted
//                m  level 1 menu id
//                i  level 2 menu id
//                s  level 3 menu id
// Returns:       Nothing
// Description:
// *****************************************************************************
function _LoseHighlight(ob, m, i, s) {
   LogUtilFunc("_LoseHighlight", "");
   if (ob) {
      ob.className = NORMAL_STYLE;
      var a = GetMenuAnchor(m, i, s);
      if (null != a) {
         a.className = NORMAL_STYLE;
      }
   }
}

// *****************************************************************************
// Function:      LoseHighlight
// Parameter:     highlightedOb  Object to remove highlight from
// Returns:       Nothing
// Description:
// *****************************************************************************
function LoseHighlight(highlightedOb) {
   LogUtilFunc("LoseHighlight", "highlightedOb=" + ((highlightedOb != null) ? highlightedOb.toString() : ""));
   if (null != highlightedOb.m) {
      var cell = GetTableCell(highlightedOb.m);
      _LoseHighlight(cell, highlightedOb.m);
      // Lose the sub highlight
      if (highlightedOb.s) {
         var row = GetTableRow(highlightedOb.m, highlightedOb.i, highlightedOb.s);
         _LoseHighlight(row, highlightedOb.m, highlightedOb.i, highlightedOb.s);
      }
      // Lose the item highlight
      var row = GetTableRow(highlightedOb.m, highlightedOb.i);
      _LoseHighlight(row, highlightedOb.m, highlightedOb.i);
   }

   highlighted.m = null;
   highlighted.i = null;
   highlighted.s = null;
}

// *****************************************************************************
// Function:      Activate
// Parameter:     element  Element to be activated
// Returns:       Nothing
// Description:
// *****************************************************************************
function Activate(element) {
   LogFunc("Activate", "element=" + element);

   if (null != element) {
      // This is illegal (apparantly activeElement is a read only attribute). However, it
      // is needed to make the menu work!!
      document.activeElement = element;
      element.focus();
   }

   if (null != highlighted.m) {
      LoseHighlight(highlighted);
   }

   if (null != element) {
      Highlight(element);
   }
}

// *****************************************************************************
// Function:      ActivateMenu
// Parameter:     m  level 1 menu id
//                i  level 2 menu id
//                s  level 3 menu id
// Returns:       Nothing
// Description:
// *****************************************************************************
function ActivateMenu(m, i, s) {
   LogFunc("ActivateMenu", "m=" + m + " i=" + i + " s=" + s);
   var element = GetMenuAnchor(m, i, s);
   if (element == null) {
      element = GetMenuAnchor(m, i);
      if (element == null) {
         element = GetMenuAnchor(m);
      }
   }

   if (element != document.activeElement) {
      // Current active element is different from the one being requested to be activated
      Activate(element);
   }
}

// *****************************************************************************
// Function:      ActivateNextMenu
// Parameter:     m  level 1 menu id
//                i  level 2 menu id
//                s  level 3 menu id
// Returns:       Nothing
// Description:
// *****************************************************************************
function ActivateNextMenu(m, i, s) {
   LogUtilFunc("ActivateNextMenu", "m=" + m + " i=" + i + " s=" + s);
   var element = GetNextMenuAnchor(m, i, s);
   Activate(element);
}

// *****************************************************************************
// Function:      ActivatePrevMenu
// Parameter:     m  level 1 menu id
//                i  level 2 menu id
//                s  level 3 menu id
// Returns:       Nothing
// Description:
// *****************************************************************************
function ActivatePrevMenu(m, i, s) {
   LogUtilFunc("ActivatePrevMenu", "m=" + m + " i=" + i + " s=" + s);
   var element = GetPrevMenuAnchor(m, i, s);
   Activate(element);
}

// *****************************************************************************
// Function:      ExtractMenuNumber
// Parameter:     element  Menu element to be split
// Returns:       A MenuNumber object
// Description:   split the 'menu_menu_m', 'menu_item_m_i' or 'menu_sub_m_i_s' into a MenuNumber object
// *****************************************************************************
function ExtractMenuNumber(element) {
   var result = new MenuNumber(null, null, null);

   if ((ID_ITEM == element.id.substring(0, ID_ITEM.length))
   || (ID_CONTAINER == element.id.substring(0, ID_CONTAINER.length))
   || (ID_TABLE_ROW == element.id.substring(0, ID_TABLE_ROW.length))) {
      var s = element.id.split("_");
      result.m = s[1];
      result.i = s[2]==null ? null : s[2];
      result.s = s[3]==null ? null : s[3];
   }
   return result;
}

// *****************************************************************************
// Function:      GetMenuType
// Parameter:     m  level 1 menu id
//                i  level 2 menu id
//                s  level 3 menu id
// Returns:       The type of the menu defined by the parameters
// Description:
// *****************************************************************************
function GetMenuType(m, i, s) {
   if ((null != s) && (!isNaN(s))) {
      return SUB_TYPE;
   }
   else if ((null != i) && (!isNaN(i))) {
      return ITEM_TYPE;
   }
   else {
      return MENU_TYPE;
   }
}

// *****************************************************************************
// Function:      GetMenuPos
// Parameter:     anchor      A menu anchor element
//                isSubMenu   Is the menu anchor a submenu
// Returns:       A point object
// Description:   If isSubMenu is true then act like we are getting the position of an item
//                Item's are different because we don't want to add the tr offset top
// *****************************************************************************
function GetMenuPos(anchor, isSubMenu) {
   LogUtilFunc("GetMenuPos", "");
   var x = 0;
   var y = 0;

   // IWB 1.2 version
   if (navigator.userAgent.toLowerCase().indexOf("20011129") != -1) {
      // a bit weird!
      // use the normal offset unless we want the position of a sub menu
      // then we have to add the offset of the div containing the anchor
      x = anchor.offsetLeft;
      y = anchor.offsetTop;
      if (isSubMenu) {
         while ((null != anchor) && (!isNaN(anchor.offsetLeft))) {
            if ("DIV" == anchor.tagName) {
               x += anchor.offsetLeft;
               y += anchor.offsetTop;
               anchor = null;
            }
            else {
               anchor = anchor.parentNode;
            }
         }
      }
      return new Point(x, y);
   }

   // Assume IWB 2.0 version
   if (isSubMenu) {
      while ((null != anchor) && (!isNaN(anchor.offsetLeft))) {
         if ("TBODY" != anchor.tagName) {
            x += anchor.offsetLeft;
            y += anchor.offsetTop;
            Log(anchor.offsetLeft + "," + anchor.offsetTop + "," + anchor.tagName);
            if (("DIV" == anchor.tagName) && (anchor.id) && (ID_CONTAINER == anchor.id.substring(0, 14))) {
               anchor = null;
            }
            else {
               anchor = anchor.parentNode;
            }
         }
         else {
            anchor = anchor.parentNode;
         }
      }
   }
   else {
      while ((null != anchor) && (!isNaN(anchor.offsetLeft))) {
         if (("TR" != anchor.tagName) && ("TBODY" != anchor.tagName)) {
            Log(anchor.offsetLeft + "," + anchor.offsetTop + "," + anchor.tagName);
            x += anchor.offsetLeft;
            y += anchor.offsetTop;
         }
         anchor = anchor.parentNode;
      }
   }
   return new Point(x, y);
}

// *****************************************************************************
// Function:      Point
// Parameter:     x  x coordiante of point
//                y  y coordinate of point
// Returns:       Nothing
// Description:
// *****************************************************************************
function Point(x, y) {
   LogUtilFunc("Point", "");
   this.x = x;
   this.y = y;
}

// *****************************************************************************
// Function:      ExecuteMenu
// Parameter:     m  level 1 menu id
//                i  level 2 menu id
//                s  level 3 menu id
// Returns:       Nothing
// Description:
// *****************************************************************************
function ExecuteMenu(m, i, s) {
   LogFunc("ExecuteMenu", "m=" + m + " i=" + i + " s=" + s);
   var menu = GetMenu(m, i, s);
   var posWin = "";
   if(isDesktop){
      var coords  = calculateWindowCoords();
      posWin = "left="+coords.posX+",top="+coords.posY+",";
   }

   if (null == menu) {
      return;
   }

   HideAll();
   menuDropped.m = null;
   menuDropped.i = null;
   menuDropped.s = null;

   // Set focus away from menu as this is now hidden (if level 1 menu still do
   // this to ensure consistancy
   FocusNextFocusable();

   LogEvent("ExecuteMenu", "Operation = " + menu.op + ", Event = " + menu.evt);
      ////alert("Execute menu item\nOperation = " + menu.op + "\nEvent = " + menu.evt + "\nappId = " + menu.appId + "\nKey Type = " + menu.keyType);
   if ((null != menu.appId) && (menu.appId.length > 0)) {
      hdtLaunchApplication(menu.appId,menu.keyType,posWin + newWindowOptions,menu.processorName, menu.applicationName);
   }
   else {
      if (menu.closeAllReq == "true") {
         applicationCloseAll(menu.op, menu.evt);
      }
      else {
         submitEventOrOp(menu.op, menu.evt);
      }
   }
}

function ExecuteMenuAfterAppCloseAll(op, evt) {
   submitEventOrOp(op, evt);
}

// *****************************************************************************
// Function:      calculateWindowCoords
// Parameter:     none
// Returns:       position of open window relative to table
// Description:
// *****************************************************************************
function calculateWindowCoords() {
   var LOGO_LINE  = "logoTable";
   var el = document.getElementById(LOGO_LINE);
   var winWidth = 0;
   var winHeight = 0;
   var scrollbarWidth = 5; // width of sidebar
   // coordinates of element content relative to the container element.
   y = el.offsetTop;
   x = 0;

   // add space from screen sides to window
   y += window.screenY + titleBarHeight + 3;// adds window title bar height this is missed in os2, and spacer for under keydisplay line.
   x = window.screenX + scrollbarWidth;
   // Find the width and height the window will be opened at
   var optionsArray = newWindowOptions.split(",");
   for (i=0; i<optionsArray.length; i++){
      var option = optionsArray[i];
      if (option.search("innerheight") != -1) {
         var stringPos = option.search("=");
         var winHeight = parseInt(option.substr(stringPos+1));
      }
      if (option.search("innerwidth") != -1) {
         var stringPos = option.search("=");
         var winWidth = parseInt(option.substr(stringPos+1));
      }
   }
   // Add the window dimensions to the screen space around the window
   var screenHeight = screen.height;
   var screenWidth = screen.width;
   var winFinalHeight = winHeight + y;
   var winFinalWidth = winWidth + x + scrollbarWidth;    // add width of right sidebar
   //check the window will stay vertically within the screen area
   if (winFinalHeight > screenHeight){
      y = (screenHeight - winHeight);
   }
   if (winFinalHeight < 0 || y < 0){
      y = 0;
   }

   //check the window will stay horizontally within the screen area
   if (winFinalWidth > screenWidth){
      x = (screenWidth - winWidth);
   }
   if (winFinalWidth < winWidth){
      x = 0;
   }
   return new Coords(x,y);
}

// *****************************************************************************
// Function:      Coords
// Parameter:     x,y  Position of ActiveKeyLine
// Returns:       Nothing
// Description:
// *****************************************************************************

function Coords(x,y) {
   this.posX = x;
   this.posY = y;
}

// *****************************************************************************
// Function:      NullEvent
// Parameter:     None
// Returns:       Nothing
// Description:
// *****************************************************************************
function NullEvent() {
   LogEvent("NullEvent", "");
}

// *****************************************************************************
// Function:      MouseClick
// Parameter:     m  level 1 menu id
//                i  level 2 menu id
//                s  level 3 menu id
// Returns:       Nothing
// Description:
// *****************************************************************************
function MouseClick(event, m, i, s) {
   LogEvent("MouseClick", "m=" + m + " i=" + i + " s=" + s + " menuDropped=" + menuDropped.toString() + " activeElement=" + document.activeElement);

   var menuType = GetMenuType(m, i, s);
   var prevActive = document.activeElement;

   // In IWB 2.0 (not IE or Mozilla) clicking both left & right mouse buttons together on the menu bar
   // caused a new browser window to be launched. Following condition prevents this.

   if (event.which == 2) {
      event.stopPropagation(); // stops the event from bubbling
      return;
   }

   // This is illegal (apparantly activeElement is a read only attribute). However, it
   // is needed to make the menu work!!
   document.activeElement = GetMenuAnchor(m, i, s);

   // if this is a menu that we should execute (has no menus or sub menus)
   if ((SUB_TYPE == menuType) || ((ITEM_TYPE == menuType) && (!HasItems(m, i))) || ((MENU_TYPE == menuType) && (!HasItems(m)))) {
      HideAll();
      menuDropped.m = null;
      menuDropped.i = null;
      return;
   }

   if (ITEM_TYPE == menuType) {
      if (HasItems(m, i)) {
         if (null == menuDropped.i) {
            ShowSub(m, i);
            ActivateMenu(m, i, 0);
         }
         else {
            HideSub(m, i);
            menuDropped.m = m;
            menuDropped.i = null;
            ActivateMenu(m, i);
         }
      }
      return;
   }

   if (MENU_TYPE == menuType) {
      if (HasItems(m)) {
         ShowMenu(m);
         ActivateMenu(m, 0);
         ShowMenu(m);
      }
   }
}

// *****************************************************************************
// Function:      MenuBlur
// Parameter:     m  level 1 menu id
//                i  level 2 menu id
//                s  level 3 menu id
// Returns:       Nothing
// Description:
// *****************************************************************************
function MenuBlur(m, i, s) {
   LogEvent("MenuBlur", "m=" + m + " i=" + i + " s=" + s + " menuDropped=" + menuDropped,toString() + " activeElement=" + document.activeElement);

   var menuType = GetMenuType(m, i, s);
   var active = document.activeElement;
   Log("active=" + active);

   if (!IsMenuAnchor(active)) {
      while ((null != active) && (!IsTableRow(active))) {
         active = active.parentNode;
      }
      if (null != active) {
         var rowNumber = ExtractMenuNumber(active);
         active = GetMenuAnchor(rowNumber.m, rowNumber.i, rowNumber.s);
      }
   }

   // See if focus has left window. This happens when an item we think has focus
   // appears to still have focus even though we are processing a blur event for it.
   myId = ID_ITEM + m;
   if (!isNaN(i)) {
      myId = myId + "_" + i;
   }

   if (!isNaN(s)) {
      myId = myId + "_" + s;
   }

   Log("myId=" + myId);
   if ((null == active) || (!IsMenuAnchor(active)) || (myId == active.id)) {
      if (menuDropped.i != null && menuDropped.s != null) {
         Log("Hiding sub menu");
         HideSubMenu(menuDropped.i, menuDropped.s);
      }

      if (menuDropped.m != null) {
         Log("Hiding dropped menu");
         HideAll();
      }

      menuDropped.i = null;
      menuDropped.s = null;
      return;
   }

   // Active element is a menu
   Log("active.id=" + active.id);
   var menuID = ExtractMenuNumber(active);
   if (m == menuID.m) {
      // Same menu
      if (SUB_TYPE == menuType) {
         // If a sub menu is losing focus
         if (i != menuID.i) {
            HideSub(m, i);
         }
      }
   }
   else {
      // Different menu
      // If a sub loses focus, hide it
      if (SUB_TYPE == menuType) {
         HideSub(m, i);
         menuDropped.i = null;
      }
   }
}

// *****************************************************************************
// Function:      GetHotkeyFromKeyCode
// Parameter:     keyCode  The key code to be converted
// Returns:       The hotkey (or null)
// Description:
// *****************************************************************************
function GetHotkeyFromKeyCode(keyCode) {
   LogUtilFunc("GetHotKeyFromKeyCode", keyCode);

   // Convert numeric key pad numbers for processing
   if (keyCode >= KEY_NUMPAD_0 && keyCode <= KEY_NUMPAD_9) {
      keyCode = keyCode - KEY_NUMPAD_0 + KEY_0;
   }

   // Windows and OS/2 require different processing here
   // Windows is not supported live but code is in here to allow testing against
   // the Windows build of Mozilla
   if (thisOS.substring(0, os2.length) == os2) {
      var x = keyCode % 256;
      if ((x >= 48) && (x <= 48+10)) {
         return ZERO_TO_NINE.charAt(x - 48);
      }
      else if ((x >= 97) && (x <= 97+26)) {
         // lower case letter found
         return A_TO_Z.charAt(x - 97);
      }
      else if ((x >= 65) && (x <= 65+26)) {
         // upper case letter found
         return A_TO_Z.charAt(x - 65);
      }
   }
   else {
      var c = String.fromCharCode(keyCode);
      if ((A_TO_Z.indexOf(c) > -1) || (ZERO_TO_NINE.indexOf(c) > -1)) {
         return c;
      }
   }

   return null;
}

// *****************************************************************************
// Function:      OtherKeyUp
// Parameter:     keyCode  code of the key pressed
//                m        level 1 menu id
//                i        level 2 menu id
//                s        level 3 menu id
// Returns:       Nothing
// Description:
// *****************************************************************************
function OtherKeyUp(keyCode, m, i, s) {
   LogFunc("OtherKeyUp", "keyCode=" + keyCode + " m=" + m + " i=" + i + " s=" + s + " menuDropped=" + menuDropped.toString() + " activeElement=" + document.activeElement);

   var c = GetHotkeyFromKeyCode(keyCode);
   if (null == c) {
      Log("Not a valid hotkey, keyCode=" + keyCode);
      return;
   }

   Log("Hotkey Pressed=" + c);
   // get the menu we pressed a key on
   var menu = GetMenu(m, i, s);
   if (null == menu) {
      return;
   }

   var items = null;
   // store the list of items in items
   // if the menu is a top level menu, use its items (only if dropped)
   if (MENU_TYPE == menu.type) {
      if (null != menuDropped.m) {
         items = GetMenu(m).items;
      }
   }
   // if the menu is a menu item, use its parent's items
   else if (ITEM_TYPE == menu.type) {
      items = GetMenu(m).items;
   }
   // if the menu is a sub menu item, use its parent's items
   else if (SUB_TYPE == menu.type) {
      items = GetMenu(m, i).items;
   }

   if (null == items) {
      return;
   }
   menu = null;

   // search for a hotkey match
   for (var x = 0; x < items.length; x++) {
      if (c == items[x].hotkey) {
         menu = items[x];
         if (ITEM_TYPE == menu.type) {
            // store the item index in case we need to show a sub menu
            i = x;
         }
         else if (SUB_TYPE == menu.type) {
            s = x;
         }
         break;
      }
   }

   if (null == menu) {
      return;
   }

   // Menu item will either be a menu item or a sub menu item
   if ((ITEM_TYPE == menu.type) && (HasItems(m, i))) {
      // if we should show a sub menu then do so
//      if (duplicateKeyEvent == false) {
         if(1) {
         ShowSub(m, i);
         menuDropped.i = i;
         ActivateMenu(m, i, 0);
         duplicateKeyEvent = true;
      }
      else {
         duplicateKeyEvent = false;
      }
   }
   else {
      // else we are an item or a sub item with no sub menu
      // and should be executed
      HideAll();
      menuDropped.m = null;
      menuDropped.i = null;
      ActivateMenu(0);        // Prevents loss of focus
      ExecuteMenu(m, i, s);
   }
}

// *****************************************************************************
// Function:      FocusNextFocusable
// Parameter:     None
// Returns:       Nothing
// Description:   Focuses on the next TABBABLE item.
//                Warning! This may not be the next item in the browser's tab sequence.
// *****************************************************************************
function FocusNextFocusable() {
   LogFunc("FocusNextFocusable", "");
   var all = null;
   all = document.getElementsByTagName("*");

   var next = null;
   var endOfMenu = document.getElementById("endOfMenu");
   // find the position of end of menu
   for (var x = 0; (null == next) && (x < all.length); x++) {
      var el = all[x];

      if (el == endOfMenu) {
         // find the next tabbable element
         for (var y = 0; (null == next) && (y < all.length); y++) {
            el = all[(y + x + 1) % all.length];
            for (var tagNames = 0; tagNames < TABBABLE.length; tagNames++) {
               if ((el.type != "hidden") && (!el.disabled))  {
                  if (TABBABLE[tagNames] == el.tagName) {
                     next = el;
                     break;
                  }
               }
            }
         }
      }
   }

   if (null != next) {
      HideAll();
      menuDropped.m = null;
      menuDropped.i = null;
      LogFunc("FocusNextFocusable", "setting focus to " + next.tagName + ", id=" + next.id);
      next.focus();
   }
}

// *****************************************************************************
// Function:      FocusLastFocusable
// Parameter:     None
// Returns:       Nothing
// Description:   Focuses on the last TABBABLE item.
// *****************************************************************************
function FocusLastFocusable() {
   LogFunc("FocusLastFocusable", "");
   var all = null;
   all = document.getElementsByTagName("*");

   var last = null;
   // find the position of 'last' element in document
   for (var x = all.length - 1; (null == last) && (x > 0); x--) {
      var el = all[x];

      LogFunc("FocusLastFocusable el.id ", el.id);
      for (var tagNames = 0; tagNames < TABBABLE.length; tagNames++) {
         if ((TABBABLE[tagNames] == el.tagName) && (el.type != "hidden") && (!el.disabled)) {
            last = el;
            break;
         }
      }
   }

   if (null != last) {
      HideAll();
      menuDropped.m = null;
      menuDropped.i = null;
      LogFunc("FocusLastFocusable", "setting focus to " + last.tagName + ", id=" + last.id);
      last.focus();
   }
}

// *****************************************************************************
// Function:      KeyUp
// Parameter:     e  The key up event
// Returns:       Nothing
// Description:   Handles the key presses.
// *****************************************************************************
function KeyUp(e) {
   var menuID  = ExtractMenuNumber(e.target);
   var m       = menuID.m;
   var i       = menuID.i;
   var s       = menuID.s;
   var isShift = e.shiftKey;
   var hotkey  = e.altKey;
   var keyCode = e.keyCode;

   LogFunc("KeyUp", "isShift=" + isShift + " hotkey=" + hotkey + " keyCode=" + keyCode + " m=" + m + " i=" + i + " s=" + s + " menuDropped=" + menuDropped.toString());

   // Ignore key presses that do not originate from the menu
   if ((null == m) && (null == i) && (null == s)) {
      // Unless we have a dropped menu
      if (menuDropped.m == null) {
         return;
      }
   }

   var menuType = GetMenuType(m, i, s);

   // Check for an HTML accesskey activating a menu
   // This way only work for top level menus
   // Normal keys pressed when menu is dropped down go to OtherKeyUp method
   if (hotkey) {
      var menu = GetMenu(m);
      var c = GetHotkeyFromKeyCode(keyCode);
      Log("KeyUp hotkey, c=" + c);
      if (c != null) {
         // only hide all the menus if the hotkey was received on a different menu
         // i.e. the hotkey for a different top level menu item has been pressed
         if ((m != menuDropped.m) && (c == menu.hotkey)) {
            HideAll();
            menuDropped.m = null;
            menuDropped.i = null;
            if (HasItems(m)) {
               ShowMenu(m);
               ActivateMenu(m, 0);
            }
            else {
               // only execute the action if the hotkey matches
               if (c == menu.hotkey) {
                  ExecuteMenu(m, i, s);
                  ActivateMenu(m);
               }
            }
         }
         else {
            ActivateMenu(m, 0);
         }
      }

      return;
   }

   if (KEY_ESCAPE == keyCode) {
      Log("KeyUp esc");
      // If we press ESC on a sub menu, hide it
      if (SUB_TYPE == menuType) {
         HideSub(m, i);
         menuDropped.i = null;
         ActivateMenu(m, i);
      }
      // If we press ESC on an item, hide it
      else if (ITEM_TYPE == menuType) {
         HideMenu(m);
         menuDropped.m = null;
         menuDropped.i = null;
         ActivateMenu(m);
      }
      return;
   }

   if (KEY_DOWN == keyCode) {
      Log("KeyUp down");
      if (MENU_TYPE == menuType) {
         if ((null != menuDropped.m) && (m != menuDropped.m)) {
            HideAll();
            menuDropped.m = null;
            menuDropped.i = null;
         }
         // If we press down on a menu
         if (HasItems(m)) {
            // If the menu has items
            ShowMenu(m);
            ActivateMenu(m, 0);
         }
      }
      else if (ITEM_TYPE == menuType) {
         // If we press down on a menu item
         ActivateNextMenu(m, i);
      }
      else if (SUB_TYPE == menuType) {
         // If we press down on a sub menu item
         ActivateNextMenu(m, i, s);
      }
      return;
   }

   if (KEY_UP == keyCode) {
      Log("KeyUp up");
      if (MENU_TYPE == menuType) {
         if ((null != menuDropped.m) && (m != menuDropped.m)) {
            HideAll();
            menuDropped.m = null;
            menuDropped.i = null;
         }
         // If we press up on a menu
         if (HasItems(m)) {
            // If the menu has items
            ShowMenu(m);
            ActivateMenu(m, numSubsArray[m].length-1);
         }
      }
      else if (ITEM_TYPE == menuType) {
         // If we press up on a menu item
         ActivatePrevMenu(m, i);
      }
      else if (SUB_TYPE == menuType) {
         // If we press up on a sub menu item
         ActivatePrevMenu(m, i, s);
      }
      return;
   }

   if (KEY_RIGHT == keyCode) {
      Log("KeyUp right");
      if (MENU_TYPE == menuType) {
         // If we press right on a menu
         if (null == menuDropped.m) {
            // If there is no menu dropped, simply move on to next menu
            ActivateNextMenu(m);
         }
         else {
            // Hide the currently dropped menu, and move to next
            HideMenu(menuDropped.m);
            var next = 1 + parseInt(m);
            Log("next:" + next);
            if (next >= numSubsArray.length) {
               next = 0;
            }
            if (HasItems(next)) {
               ShowMenu(next);
            }
            ActivateMenu(next, 0);
         }
      }
      else if (ITEM_TYPE == menuType) {
         // If we press right on a menu item
         if (HasItems(m, i)) {
          //  if (duplicateKeyEvent == false) {
               if(1) {
               // If there are sub items, show them
               ShowSub(m, i);
               ActivateMenu(m, i, 0);
               duplicateKeyEvent = true;
            }
            else {
               duplicateKeyEvent = false;
            }
         }
         else {
            // Hide the currently dropped menu, and move to next
            HideMenu(menuDropped.m);
            var next = 1 + parseInt(m);
            if (next >= numSubsArray.length) {
               next = 0;
            }
            if (HasItems(next)) {
               ShowMenu(next);
            }
            ActivateMenu(next, 0);
         }
      }
      else if (SUB_TYPE == menuType) {
         // If we press right on a sub menu item
         HideAll();
         var next = 1 + parseInt(m);
         if (next >= numSubsArray.length) {
            next = 0;
         }
         if (HasItems(next)) {
            ShowMenu(next);
         }
         ActivateMenu(next);
         ActivateMenu(next, 0);
      }
      return;
   }

   if (KEY_LEFT == keyCode) {
      Log("KeyUp left");
      if ((MENU_TYPE == menuType) || (ITEM_TYPE == menuType)) {
         // If we press left on a menu
         if (null == menuDropped.m) {
            // If there is no menu dropped, simply move on to next menu
            ActivatePrevMenu(m);
         }
         else {
            // Hide the currently dropped menu, and move to next
            HideMenu(m);
            var next = parseInt(m) - 1;
            if (next < 0) {
               next = numSubsArray.length - 1;
            }
            if (HasItems(next)) {
               ShowMenu(next);
            }
            ActivateMenu(next, 0);
         }
      }
      else if (SUB_TYPE == menuType) {
         // If we press left on a sub menu item
         HideSub(m, i);
         menuDropped.i = null;
         ActivateMenu(m, i);
      }
      return;
   }

   if (KEY_TAB == keyCode) {
      Log("KeyUp tab");
      if (null != menuDropped.m) {
         HideAll();
         menuDropped.m = null;
         menuDropped.i = null;
      }
      if (isShift) {
         if (MENU_TYPE == menuType) {
            FocusLastFocusable();
         }
      }
      else {
         FocusNextFocusable();
      }
      return;
   }

   // only allow OtherKeyUp if a menu is currently dropped
   // key presses should be ignored otherwise
   if (null != menuDropped.m) {
      OtherKeyUp(keyCode, m, i, s);
   }
}

// *****************************************************************************
// Function:      MenuFocus
// Parameter:     m  level 1 menu id
//                i  level 2 menu id
//                s  level 3 menu id
// Returns:       Nothing
// Description:   Activates the menu that is receiving focus.
// *****************************************************************************
function MenuFocus(m, i, s) {
   LogFunc("MenuFocus", "m=" + m + " i=" + i + " s=" + s + " menuDropped=" + menuDropped.toString() + " activeElement=" + document.activeElement);
   ActivateMenu(m, i, s);
}

// *****************************************************************************
// Function:      MouseDown
// Parameter:     e  the Gecko event object
// Returns:       Nothing
// Description:   Passes a mousedown event to the generic keyup handler.
//                If the mouse is pressed down on an item that is not part of the
//                menu, hide all the menus.
// *****************************************************************************
function MouseDown(e) {

   ////domDebug("MouseDown target=" + e.target + ", currentTarget=" + e.currentTarget + ", cancelable=" + e.cancelable);
   var element = e.target;

   //Fix for IWB 2.0.1. When click on text returns target of javascript:NullEvent(). In the old verssion it returned Text object
   //Therefore set element to child of target to set it back to to a Text object.
   if (element == "javascript:NullEvent()"){
   		element = element.firstChild;
   }
   var menuElement = null;
   lastActive = document.activeElement;
   document.activeElement = element;

   ////domDebug("MouseDown element=" + element);
   // If we have a mouse down on any element other than a menu link, the hotkey in the menu link
   // or a menu table row, we have clicked off the menu. therefore, hide the menu

   while ((null != element) && (menuElement == null)) {
      if ((null != element.id)) {
         if (IsTableRow(element)) {
            ////domDebug("MouseDown Found a table row");
            var rowNumber = ExtractMenuNumber(element);
            var lastRowNumber = ExtractMenuNumber(lastActive);
            var menu = GetMenu(rowNumber.m, rowNumber.i, rowNumber.s);
            ////domDebug("MouseDown lastActive.id=" + lastActive.id + ", menu=" + menu + ", op=" + menu.op + ", evt=" + menu.evt + ", appId=" + menu.appId);
            if (menu.op.length == 0 && menu.evt.length == 0 && menu.appId.length == 0) {
               ////domDebug("MouseDown rowNumber.m=" + rowNumber.m + ", lastRowNumber.m=" + lastRowNumber.m + "rowNumber.i=" + rowNumber.i + ", lastRowNumber.i=" + lastRowNumber.i + "rowNumber.s=" + rowNumber.s + ", lastRowNumber.s=" + lastRowNumber.s);
               if (rowNumber.m !== lastRowNumber.m) {
                  ////domDebug("MouseDown hiding all");
                  HideAll();
               }
               else if (rowNumber.m == lastRowNumber.m && rowNumber.i != lastRowNumber.i) {
                  ////domDebug("MouseDown hiding submenu");
                  HideSub(lastRowNumber.m, lastRowNumber.i);
                  ActivateMenu(rowNumber.m, rowNumber.i);
               }
            }
            menuElement = GetMenuAnchor(rowNumber.m, rowNumber.i, rowNumber.s);

            if (menu.appId != "signoff") {
               // This causes a Javascript exception. Don't know why but allows the real Mouse click event to function
               // Needs sorting out
               var mouseEvent = new Event();
            }
            else {
               // We need to really do signoff when row is clicked for some reason
               ExecuteMenu(rowNumber.m, rowNumber.i);
            }
         }
         else if (IsMenuHotkey(element)) {
            var rowNumber = ExtractMenuNumber(element.parentNode);
            var lastRowNumber = ExtractMenuNumber(lastActive);
            var menu = GetMenu(rowNumber.m, rowNumber.i, rowNumber.s);
            ////domDebug("MouseDown lastActive.id=" + lastActive.id + ", menu=" + menu + ", op=" + menu.op + ", evt=" + menu.evt + ", appId=" + menu.appId);
            if (menu.op.length == 0 && menu.evt.length == 0 && menu.appId.length == 0) {
               ////domDebug("MouseDown rowNumber.m=" + rowNumber.m + ", lastRowNumber.m=" + lastRowNumber.m + "rowNumber.i=" + rowNumber.i + ", lastRowNumber.i=" + lastRowNumber.i + "rowNumber.s=" + rowNumber.s + ", lastRowNumber.s=" + lastRowNumber.s);
               if (rowNumber.m !== lastRowNumber.m) {
                  ////domDebug("MouseDown hiding all");
                  HideAll();
               }
               else if (rowNumber.m == lastRowNumber.m && rowNumber.i != lastRowNumber.i) {
                  ////domDebug("MouseDown hiding submenu");
                  HideSub(lastRowNumber.m, lastRowNumber.i);
                  ActivateMenu(rowNumber.m, rowNumber.i);
               }
            }
            menuElement = GetMenuAnchor(rowNumber.m, rowNumber.i, rowNumber.s);

            // This causes a Javascript exception. Don't know why but allows the real Mouse click event to function
            // Needs sorting out
            var mouseEvent = new Event();
         }
         else if (IsMenuAnchor(element)) {
            Log("MouseDown Found an anchor");
            menuElement = element;
         }
      }
      element = element.parentNode;
   }

   if (null == menuElement) {
      HideAll();
      menuDropped.m = null;
      menuDropped.i = null;
   }
}

// *****************************************************************************
// Function:      getOnClickHandlerFor
// *****************************************************************************
function getOnClickHandlerFor(href) {
   //alert ("typeof isAlreadySubmitted is:" + (typeof isAlreadySubmitted));
   if (typeof isAlreadySubmitted == "undefined") {
      return ""; //this is a desktop page, leave the clicks alone
   }
   else if (href=="javascript:NullEvent()") {
      return " onclick=\"return false;\""; // never actually action nullEvents
   }
   else {
      return " onclick=\"return !isAlreadySubmitted();\""; //a standard application menu
   }
}

// *****************************************************************************
// Function:      hdtShowMenu
// Description:   Shows the hmenu, called when the desktop page has loaded.
// *****************************************************************************
function hdtShowMenu(){
   menuId = document.getElementById("HMenuTable");
   menuId.style.visibility="visible";
}

// *****************************************************************************
// Function:      IsMenuHotkey
// Parameter:     Element. The element to be checked
// Returns:       Returns true if the element is an underlined hotkey.
// Description:
// *****************************************************************************
function IsMenuHotkey(element) {
   LogUtilFunc("IsMenuHotkey", "");
   if (null == element) {
      return false;
   }
   return (element.className == "hMenuUnderline");
}
