/*****************************************************************************
 *   Module : keyController.js     
 *   Description: This file contains the functions necessary to acquire and
 *                set active keys, and display the modified key list to the
 *                user.
 *                Note: The initialise() method should be called before any
 *                      of the other methods.
 * 
 *   Traceability to design : Represents the KeyController class.
 *
 *   Modification History:
 *   Author               Date            Version              Reason
 *   D.Palmer           09-Jan-01         initial                -    
 *   A.Watson     26-Feb-01     0.1        Adapted for real desktop
 *   A.Watson     19-Apr-01     0.2        Put description back into 
 *                   option value        
 *   A.Watson     25-Apr-01     0.3        Change nbsp character for Mozilla  
 *   A.Watson     27-Jul-01     ST0138       Ensure spacing in Active Key List
 *                       is correct on Mozilla              
 ****************************************************************************/


   // Values taken by typeId.
   var NoKeySelected = -1;
   var Product       = 0;
   var Customer      = 1;
   var Card          = 2;
   var Staff         = 3;
   var Portfolio     = 4;
   
   // NewKey attribute used by Key Sharing functions
   var selectNewKey = false;
   // Holds the maximum number of keys associated with each type.              
   var hksMAX_KEYS = 5;

   // Define keyType attributes.
   var typeId      = new Array(Product, Customer, Card, Staff, Portfolio);
   var description = new Array("product", "customer", "card", "staff", "portfolio");

   var fieldSeparator = "::";

   // Holds the different groups of keys. 
   var hksKeyTypes = new Array(typeId.length);

    // Holds the currently Active key Type.
    var hksActiveKeyType = null;
   
   /* Drop down option indicating that a key has not been selected.
    * Note: text consists of keyValue + keyDescription; value consists
    *       of keyId + keyValue + keyType.
    */
    var noKeyOption = 
            new Option("New Key" +
                       "\xA0\xA0\xA0\xA0\xA0\xA0\xA0\xA0\xA0\xA0" +
                       "\xA0\xA0\xA0\xA0\xA0\xA0\xA0\xA0\xA0\xA0" +
                       "\xA0\xA0\xA0\xA0\xA0\xA0\xA0\xA0\xA0\xA0" +
                       "\xA0\xA0\xA0\xA0\xA0\xA0\xA0\xA0\xA0\xA0" +
                       "\xA0\xA0\xA0\xA0\xA0\xA0\xA0\xA0\xA0\xA0" +
                       "\xA0\xA0\xA0", 
                       NoKeySelected + fieldSeparator + " " + fieldSeparator + " " + fieldSeparator + " ");      
                            
   /*
    *  This function places the application in its start up state.
    */
    function hksInitialise() { 
        //alert("Initialise");
        // Clear ActiveKeyType
        hksActiveKeyType = null;
                                     
        // Populate the keyTypes. 
        for (var i = 0; i < hksKeyTypes.length; i++) {
            hksKeyTypes[i] = 
              new keyType(typeId[i], description[i], new Array()); //hksMAX_KEYS));
        } 
                   
        // Initialise drop down list.
        document.keyPanel.displayedKeyValue.options[0] = noKeyOption;
        document.keyPanel.displayedKeyValue.options[0].selected = true;          
    }


   /*  
    *  This function returns the current active key.
    */
    function hksGetActiveKey() {

        if (hksActiveKeyType != null) {
            return hksActiveKeyType.getTopKey();
        }
        else {
            return null;
        }
    }


    /*
     *  This function sets the Active Key to the keyType for which theKey is a member. 
     */
     function hksSetActiveKey(theKey) {
          
         if (theKey != null) {
             for ( i = 0; i < hksKeyTypes.length; i++) {      
                 if (hksKeyTypes[i].typeId == theKey.typeId) {
                                      
                     hksActiveKeyType = hksKeyTypes[i];
                     
                     hksActiveKeyType.setTopKey(theKey);
                     
                     // Active key has been identified.
                     break;
                 }
                 else if (theKey.typeId == NoKeySelected) {                       
                     hksActiveKeyType = null;
                     break;
                 }
                 else { 
                     // key not found look at next one.
                     hksActiveKeyType = null;
                 }                
            }     
         }   
         else {
             // the parameter theKey is null.
             hksActiveKeyType = null;
         }             
           
         // Display reordered keys.
         hksUpdateKeyList();             
    }    
   /* 
    * This function is used by the XML Messaging Applet to update the key list 
    */
    function hksAddKey(keyId, keyValue, keyDescription, keyType, isActiveKey) {
        
        // Creates a new key object providing the keyId is not null
        var theKey = (keyId==null) ? null : (new key(keyId, keyValue, keyDescription, keyType)); 
     
        // Checks whether key is to be set as the active key
        if(isActiveKey == 'true') {
            selectNewKey = false;
            hksSetActiveKey(theKey);  
        }
        else {
            // var activeKey = hksGetActiveKey();
            
            if (theKey != null) {
                for (i = 0; i < hksKeyTypes.length; i++) {      
                    if (hksKeyTypes[i].typeId == theKey.typeId) {
                        var keysForType = hksKeyTypes[i];
                        
                        // Checks to see if key is the active key, if so it is added as the top key
                        if(keysForType == hksActiveKeyType) {
                           var topKey = hksActiveKeyType.getTopKey();
                           keysForType.addKey(theKey);
                           keysForType.setTopKey(topKey);
                           
                        }
                        else {
                           // Else, key is added to the list
                           keysForType.addKey(theKey);
                           break;
                        }
                    }
                    else if (theKey.typeId == NoKeySelected) {                       
                        break;
                    }
                }     
            }
            var activeKey = hksGetActiveKey();

            hksUpdateKeyList();          
        }
    }
   /*
    * This function is used by the XML Messaging Applet to update the key list.
    * Called when the key received by the applet does not have a description.
    */ 
    function hksAddKeyWithoutDesc(keyId, keyValue, keyType, isActiveKey) {
        
        var keyDescription = '';
        hksAddKey(keyId, keyValue, keyDescription, keyType, isActiveKey);
    }

   /*
    *  This function removes theKey from the ActiveKeyType key element.
    *  If theKey is not in the key list do nothing.
    */
    function hksRemoveKey(theKey) {

        if ((hksActiveKeyType != null) && 
            (keyInList(hksActiveKeyType, theKey) != NoKeySelected)) {
                hksActiveKeyType.removeKey(theKey);
        } 
    }


   /*
    *  This function sets the user selected key from the drop downlist
    *  as the active key.
    */
    function hksKeySelected(theKey) {
    
      // Holds a reference to where theKey is located in the keyList.  
      var selectedKey = null;  
      
      // Determine the selected key. 
      var index       = theKey.selectedIndex
      var selectValue = theKey.options[index].value; 
      var keyValue    = selectValue.split(fieldSeparator);    
      
      findKeyValueInList: 
      for (var i = 0; i < hksKeyTypes.length; i++) {
         for (var j = 0; j < hksMAX_KEYS; j++) {   
            if (hksKeyTypes[i].keys[j] != null) {
               if (keyValue[0] == hksKeyTypes[i].keys[j].typeId && 
                   keyValue[1] == hksKeyTypes[i].keys[j].value && 
                   keyValue[2] == hksKeyTypes[i].keys[j].type && 
                   keyValue[3] == hksKeyTypes[i].keys[j].description)
               {
                  selectedKey = hksKeyTypes[i].keys[j];
                  break findKeyValueInList;
               }
            }      
         }  
      }
      hksSetActiveKey(selectedKey);         
   }


   /*
    *  This function clears the dropdown key list and rebuilds it according to
    *  the order they are stored in hksKeyTypes. If an active key does not 
    *  exist, a key indicating that, a key is not currently selected, is 
    *  placed at the top of the list.    
    */
    function hksUpdateKeyList() { 

        // Holds the key currently indexed key with hksKeyTypes list.
        var indexedKey = null;
                   
        // Indicates whether the key should be set as the selected option.
        var keySelected = null;

        var optionPosition = 0;         
                                    
        var activeKey = hksGetActiveKey(); 
    
        // Remove all the options from the drop down list.
        document.keyPanel.displayedKeyValue.options.length = 0;
           
        // The no key option is always first
        document.keyPanel.displayedKeyValue.options[0] = noKeyOption; 
        document.keyPanel.displayedKeyValue.options[0].selected = true; 
            
        optionPosition++; 
        myRegExp=/\s/g;        
        // Build the updated drop down list.
        for (var i = 0; i < hksKeyTypes.length; i++) {                         
           for (var j = 0; j < hksKeyTypes[i].keys.length; j++) {
              indexedKey =  hksKeyTypes[i].keys[j];

              if (indexedKey != null) { 
                 document.keyPanel.displayedKeyValue.options[optionPosition] = 
                    new Option(indexedKey.value.replace(myRegExp,"\xA0")  + "\xA0" + indexedKey.description.replace(myRegExp,"\xA0"),
                               indexedKey.typeId + fieldSeparator + indexedKey.value + fieldSeparator + indexedKey.type + fieldSeparator + indexedKey.description);
                 document.keyPanel.displayedKeyValue.options[optionPosition].selected = (indexedKey == activeKey)? true:false; 
  
                 // Checks whether the new key option should be selected.
                 // This may be the case when the Messaging applet has received a key that should not be
                 // made the active key.
                 if(selectNewKey) {
                    document.keyPanel.displayedKeyValue.options[0].selected = true; 
                    selectNewKey = false;
                 }
                 optionPosition++;
              }  
           }
        } 
    }


   /*
    *  This function clears down the active key.
    */
    function clearActiveKey() {
    
        hksActiveKeyType = null;
        
        // Clear the displayed keys.
        hksUpdateKeyList();           
    }

