// This JavaScript file contains all functions relating to the handling of
// keyboard activity (with the exception of menu related functionality).

// Define some global variables
// named keyhandler_### to reduce potential namespace conflicts
var keyhandler_isIE = (!document.all) ? 0 : 1;

var keyhandler_f5          = 116;   // 't'
var keyhandler_f6          = 117;   // 'u'
var keyhandler_f8          = 119;   // 'w'
var keyhandler_f9          = 120;   // 'x'
var keyhandler_f10         = 121;   // 'y'
var keyhandler_f11         = 122;   // 'z'
var currentPage            = null;

var keyhandler_backspace   = 8;
var keyhandler_enter       = 13;
var keyhandler_escape      = 27;
var keyhandler_home        = 36;
var keyhandler_cursorLeft  = 37;
var keyhandler_cursorUp    = 38;
var keyhandler_cursorRight = 39;
var keyhandler_cursorDown  = 40;

var keyhandler_D           = 68;
var keyhandler_E           = 69;
var keyhandler_H           = 72;
var keyhandler_I           = 73;
var keyhandler_N           = 78;
var keyhandler_P           = 80;
var keyhandler_R           = 82;
var keyhandler_S           = 83;

var desk                   = 48;
var desknum                = 96; // '`'

var keyhandler_d           = 100; // lowercase
var keyhandler_e           = 101; // lowercase
var keyhandler_j           = 106; // lowercase
var keyhandler_n           = 110; // lowercase
var keyhandler_p           = 112; // lowercase
var keyhandler_r           = 114; // lowercase
var keyhandler_s           = 115; // lowercase
var keyhandler_u           = 117; // lowercase

var customCtxMenu          = true;

var duplicateKeyUp         = false;

// handleKeyUp
// Handles all keyUp events, calling the appropriate function for keys
// we want to handle specifically
function handleKeyUp(event) {
   var keyCode = (!keyhandler_isIE) ? event.which : event.keyCode;
    switch (keyCode) {
      case keyhandler_f8:
         handleKeyDownF8();
         break;
      
      case keyhandler_f10:
         if (!keyhandler_isIE) {
            keyhandler_blockKey(event);
            if (!event.altKey && !event.ctrlKey && event.shiftKey){
               event.stopPropagation(); // stops the event from bubbling
               showContextMenu();
            }
         }
         break;
       
    }
    return true;
}

// handleKeyDown
// Handles all keyDown events, calling the appropriate function for keys
// we want to handle specifically
function handleKeyDown(event, posOpName, posEventName, negOpName, negEventName) {
   var keyCode = (!keyhandler_isIE) ? event.which : event.keyCode;

   switch (keyCode) {

      case keyhandler_enter:
         if (!event.altKey && !event.ctrlKey && !event.shiftKey){
        		// return/enter key should invoke default positive action button
  	      	if ((posOpName != undefined) || (posEventName != undefined)) {
     	     		handleEnterAction(event, posOpName, posEventName);
         	}
         }         
         break;

      case keyhandler_escape:
         if (!event.altKey && !event.ctrlKey && !event.shiftKey){
	         // escape key should invoke default negative action button
   	      if ((negOpName != undefined) || (negEventName != undefined)) {
      	      handleEscapeAction(event, negOpName, negEventName);
         	}
         }  
         blockUnmodifiedKey(event);	// ESC = IE stop animation, next page loading    
         break;

      case keyhandler_f6:           // F6 = IE put focus of page somewhere (not visible), can then do CTRL+n
      case keyhandler_f11:          // F11 = IE full screen
         if (keyhandler_isIE) {
               blockUnmodifiedKey(event);
         }
         break;

      case keyhandler_f10:
         if (keyhandler_isIE) {
            keyhandler_blockKey(event);
            if (!event.altKey && !event.ctrlKey && event.shiftKey){
               showContextMenu();
            }
         }
         break;

      case keyhandler_backspace:    // BACKSPACE = IE previous page/back (except in text fields)
         if (keyhandler_isIE) {
            if (!event.srcElement.isContentEditable) {
               event.returnValue=false;
            }
         }
         break;
         
      case keyhandler_home:            // ALT+HOME = IE Loads browser HomePage
      case keyhandler_cursorLeft:      // ALT+LEFT = IE Previous page/back
      case keyhandler_cursorRight:     // ALT+RIGHT = IE Next Page/next
         if (keyhandler_isIE) {
            blockAltModifiedKey(event);
         }
         break;

      case keyhandler_D:            // CTRL+d = IE add page to bookmarks
      case keyhandler_E:            // CTRL+e = IE display search bar, can then do CTRL+n
      case keyhandler_H:            // CTRL+h = IE display history bar, can then do CTRL+n
      case keyhandler_I:            // CTRL+i = IE display favourites
      case keyhandler_R:            // CTRL+r = IE reload
         if (keyhandler_isIE) {
            blockCtrlModifiedKey(event);
         }
         break;

      case keyhandler_N: // allow developers to do CTRL-N from localhost
         if (keyhandler_isIE) {
         var hostNameId = document.location.hostname;
            if (!(hostNameId == "localhost"
               || hostNameId == "127.0.0.1"
               || hostNameId.match(/^[mM][gG][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9]$/))) {
                  if (isDesktop && event.ctrlKey) {
                     launchTechDets();  //launches new window to display the technical details of the session.
                  }
                  try {
	                  if (isSignOn && event.ctrlKey) {
	                     launchTechDets();  //launches new window to display the technical details of the session.
	                  }                  
	               } catch (Exception) {
	               	// ignore, isSignOn is probably undefined
	               }
                  
                  blockCtrlModifiedKey(event);
            }
            else if (!event.altKey && event.ctrlKey && !event.shiftKey) {
               customCtxMenu = !customCtxMenu;
               blockCtrlModifiedKey(event);
            }
         }
         break;

      case keyhandler_f5:              // f5 = IE reload
         if (keyhandler_isIE) {        // CTRL=f5 = IE reload not from cache
            blockUnmodifiedKey(event);
            blockCtrlModifiedKey(event);
         }
         break;

    }
}// end handleKeyDown


// handleKeyPress
// Handles all keyPress events, calling the appropriate function for keys
// we want to handle specifically
function handleKeyPress(event, id) {
   var keyCode = (!keyhandler_isIE) ? event.which : event.keyCode;
   if (keyCode == 0 && !keyhandler_isIE) {
      // In Mozilla the folowing keys generate a "which" of 0 and a keycode:
      // escape, left, right, all f-keys (even with Ctrl),
      // In this case we get the correct keycode from event.keyCode.
      // In order to stop these keys looking like their normal counterparts
      // we add 1000 to them.
      // For example f5 is which =0 keycode = 116 would look like a 't'
      // so we add 1000 to 116 to get 1116.
      keyCode = event.keyCode + 1000;
   }

   switch (keyCode) {

      case keyhandler_backspace:       // BACKSPACE = MOZ previous page/back
         if (!keyhandler_isIE) {
            if (!(event.target.type == "text" || event.target.type == "password" || event.target.type == "textarea")){
               blockUnmodifiedKey(event);
            }
         }
         break;

      case keyhandler_escape + 1000:   // ESC = MOZ stop animations, next page loading
          
      case keyhandler_f11 + 1000:      // F11 = MOZ full screen (sometimes with menu)
         if (!keyhandler_isIE) {
               blockUnmodifiedKey(event);
         }
         break;

      case keyhandler_enter:        // SHIFT Enter = MOZ Save Linked Page
         if (!keyhandler_isIE) {
            blockShiftModifiedKey(event);
         }
         break;

      case keyhandler_home + 1000:        // ALT+HOME = MOZ Loads browser HomePage
      case keyhandler_cursorRight + 1000: // ALT+RIGHT = MOZ Next Page/next
      case keyhandler_cursorLeft + 1000:  // ALT+LEFT = MOZ Previous page/back
         if (!keyhandler_isIE) {
            blockAltModifiedKey(event);
         }
         break;

      case keyhandler_d:                  // CTRL+d = MOZ add page to bookmarks
      case keyhandler_e:                  // CTRL+e = MOZ edit page
      case keyhandler_j:                  // CTRL+j = CTRL+ENTER = MOZ open link in new window
      case keyhandler_r:                  // CTRL+r = MOZ Reload Page
      case keyhandler_s:                  // CTRL+s = MOZ save page as
      case keyhandler_u:                  // CTRL+u = MOZ view source
         if (!keyhandler_isIE) {
            blockCtrlModifiedKey(event);
         }
         break;

      case keyhandler_n:                  // CTRL+n = MOZ new window
         if (!keyhandler_isIE) {
           
      // allow developers to do CTRL-N from localhost
      var hostNameId = document.location.hostname;
      if (!(hostNameId == "localhost"
      || hostNameId == "127.0.0.1"
      || hostNameId.match(/^[mM][gG][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9]$/)))
      {
         if (isDesktop && event.ctrlKey) {
          
            event.stopPropagation(); // stops the event from bubbling
            launchTechDets();  //launches new window to display the technical details of the session.
         }
         try {
	         if (isSignOn && event.ctrlKey) {
	          
	            event.stopPropagation(); // stops the event from bubbling
	            launchTechDets();  //launches new window to display the technical details of the session.
	         }         
           } catch (Exception) {
           	// ignore, isSignOn is probably undefined
           }
         
         blockCtrlModifiedKey(event);
      }
      else if (!event.altKey && event.ctrlKey && !event.shiftKey) {
                     event.stopPropagation(); // stops the event from bubbling
                     customCtxMenu = !customCtxMenu;
                     blockCtrlModifiedKey(event);
      }

         }
         break;


      case keyhandler_D:                  // CTRL+SHIFT+d = MOZ add page to bookmarks and edit
      case keyhandler_R:                  // CTRL+SHIFT+r = MOZ reload not from cache
         if (!keyhandler_isIE) {
            blockCtrlShiftModifiedKey(event);
         }
         break;

      case keyhandler_f5 + 1000:          // f5 = MOZ reload
         if (!keyhandler_isIE) {          // CTRL+f5 = MOZ reload not from cache
            blockUnmodifiedKey(event);
            blockCtrlModifiedKey(event);
         }
         break;

    }// end switch
}// end handleKeyPress


function handleContextKey(event, id) {

   elementId = id.toString();

   // prevent duplicate key up events in IWB (IWB bug - event fired twice)
//   if (!duplicateKeyUp || keyhandler_isIE) {
      if(1) {
      duplicateKeyUp = true;
      // Get key code - for IWB get remainder after dividing by 256 to get key code from event.which
      // as this also contains other information in the 1st byte
      var keyCode = (window.Event) ? event.which % 256 : event.keyCode;
      switch (keyCode) {
         case keyhandler_cursorUp :
         case 104:	//8 key on numeric keypad - should always function as cursor up
            handleKeyDownUp(id);
            break;
         case keyhandler_cursorDown :
         case 98:		//2 key on numeric keypad - should always function as cursor down
            handleKeyDownDown(id);
            break;
         case keyhandler_P:
         case keyhandler_p:
            printPage();
            break;
         case keyhandler_R:
         case keyhandler_r:
            arrangeWindows();
            hideContextMenu();
            break;
         case keyhandler_S:
         case keyhandler_s:
            closeAll();
            hideContextMenu();
            break;
         case keyhandler_escape:
            hideContextMenu();
            break;
         case desk:
            selectDesktop();
            hideContextMenu();
            break;
         case desknum:
            selectDesktop();
            hideContextMenu();
            break;
         case keyhandler_enter :
            lastCharIndex = elementId.length - 1;
            lastChar = elementId.charAt(lastCharIndex)
            if (lastChar == 0) {
               arrangeWindows();
            }
            if (lastChar == 1) {
               closeAll();
            }
            if (lastChar == 2) {
               printPage();
            }
            hideContextMenu();
            break;
         default:
            hotKeySwitchWindow(keyCode);
            break;
      }
   }
   else {
      duplicateKeyUp = false;
   }
   return true;
}

// handleKeyDownF8
// Swaps focus between the desktop window and an application window
function handleKeyDownF8() {
   if (isDesktop) {
      if (currentPage != null) {
         try {
            currentPage.focus();
         }
         catch (e) {
            // Page has been closed
            currentPage = null;
         }
      }
   }
   else {
      opener.currentPage = this;
      opener.setOpenWindowListFocus(window);
   }
}

// handleListBoxKey
// checks whether return is pressed
function handleListBoxKey(event){
   var keyCode = (window.Event) ? event.which : event.keyCode;
   switch (keyCode) {
      case keyhandler_enter:
            switchWindow();
         break;
      default:
         handleKeyDown(event);
         break;
   }
}

function keyhandler_blockKey(event){
   if (keyhandler_isIE) {
      if (!event.altKey) {
         event.keyCode=0; // if we don't set the keycode to 0 some keys
                          // e.g. F6 are still actioned. However Alt-Left
                          // is actioned if we do set to 0.Unsure of what
                          // the rules are, but alt seems to be useable as
                          // a decider.
                          // Some keys don't need the keyCode setting to 0
                          // but it doesn't seem to matter in those cases.
      }
      if (event.keyCode == keyhandler_home){
         alert("This functionality is disabled");// Adding an alert was the only way
                                        // we have found to stop the short-cut
                                        // Alt+Home. Text agreed with business.  
      }
      event.returnValue = false; // for IE this typically blocks the event
   }
   else
   {
      event.stopPropagation(); // stops the event from bubbling
      event.preventDefault();  // In Moz this blocks the Event
   }
}

/** Generic function to block a key that has no modifiers pressed, not even shift */
function blockUnmodifiedKey(event) {
   if (!event.altKey && !event.ctrlKey && !event.shiftKey) {
      keyhandler_blockKey(event);
   }
}


/** Generic function to block a key that has the SHIFT Key Pressed */
function blockShiftModifiedKey(event) {
   if (!event.altKey && !event.ctrlKey && event.shiftKey) {
      keyhandler_blockKey(event);
   }
}

/** Generic function to block a key that has the Ctrl Key Pressed */
function blockCtrlModifiedKey(event) {
   if (!event.altKey && event.ctrlKey && !event.shiftKey) {
      keyhandler_blockKey(event);
   }
}

/** Generic function to block a key that has the Ctrl and Shift Keys Pressed */
function blockCtrlShiftModifiedKey(event) {
   if (!event.altKey && event.ctrlKey && event.shiftKey) {
      keyhandler_blockKey(event);
   }
}

/** Generic function to block a key that has the Alt Key Pressed */
function blockAltModifiedKey(event) {
   if (event.altKey && !event.ctrlKey && !event.shiftKey) {
      keyhandler_blockKey(event);
   }
}


// Selects desktop from context menu
// Swaps focus to the desktop window from an application window
function selectDesktop() {
   hideContextMenu();
   if (!isDesktop) {
      opener.setOpenWindowListFocus(window);
   }
}

// handleKeyDownUp
//
function handleKeyDownUp(id) {
   var lastCtxMenu = null;
   navigatingCtxMenu = true;

   for (i = (document.links.length)-1; i >= 0; i--) {
      elementId = document.links[i].id;
      if (elementId.substring(0, 10) == "hafmenuctx") {
         if (lastCtxMenu == null) {
            lastCtxMenu = document.links[i];
         }
         if (elementId == id) {
            // find previous ctx menu element
            for (j = i - 1; j >= 0; j++) {
               previousElementId = document.links[j].id;
               if (previousElementId.substring(0, 10) == "hafmenuctx") {
                  document.links[j].focus();
                  break;
               }
            }
            if (j < 0) {
                lastCtxMenu.focus();
                break;
            }
         }
      }
   }
   navigatingCtxMenu = false;
}

// handleKeyDownDown
//
function handleKeyDownDown(id) {
   var firstCtxMenu = null;
   navigatingCtxMenu = true;

   for (i = 0; i < document.links.length; i++) {
      elementId = document.links[i].id;
      if (elementId.substring(0, 10) == "hafmenuctx") {
         if (firstCtxMenu == null) {
            firstCtxMenu = document.links[i];
         }
         if (elementId == id) {
            // find next ctx menu element
            for (j = i + 1; j < document.links.length; j++) {
               nextElementId = document.links[j].id;
               if (nextElementId.substring(0, 10) == "hafmenuctx") {
                  document.links[j].focus();
                  break;
               }
            }
            if (j >= document.links.length) {
               firstCtxMenu.focus();
               break;
            }
         }
      }
   }
   navigatingCtxMenu = false;
}

/**
/ handleEnterAction()
/
/ Determines what action to take when the enter or return keys are pressed.
/ Depends upon which control has focus and whether or not a default continue action has been
/ defined for the page.
**/
function handleEnterAction(event, posOpName, posEventName) {
   var srcEl = "";
   if (keyhandler_isIE) {
      srcEl = event.srcElement;
   }
   else {
      srcEl = event.target;
   }
   var tag = srcEl.tagName;

   if ((tag == "TEXTAREA") || (tag == "A") || (tag == "SELECT" && srcEl.size == 1)) {
      //Text area, menu, action button or combo box has focus.
      //Do nothing, standard browser functionality should occur
   }
   else {
	   if (posOpName == "null" && posEventName == "null") {
			//If neither posOpName nor posEventName have been set in the jsp we need to attempt to locate
	      //a default action button on the page.   
      	posEventName = findDefaultButton(event, posEventName);
      }
      
      if (posEventName != "null" || posOpName != "null") {
			try {
				//if the field has an onblur event perform that before submitting page. 
				//This ensures any field formatting is carried out
				srcEl.onblur();
			}
			catch (ex) {
				//do nothing. If the field has no onblur event then it doesn't need formatting.
			}
         submitEventOrOp(posOpName, posEventName);
      }
   }

}

/**
/ handleEscapeAction()
/
/ Determines what action to take when the escape key is pressed.
/ Depends upon which control has focus and whether or not a default exit action has been
/ defined for the page.
**/

function handleEscapeAction(event, negOpName, negEventName) {
   var srcEl = "";
   if (keyhandler_isIE) {
      srcEl = event.srcElement;
   }
   else {
      srcEl = event.target;
   }
   var tag = srcEl.tagName;
   var id = srcEl.id;

   if ((id.substring(0,8) == "menuItem" && id.length > 10) || (tag == "SELECT" && srcEl.size == 1)) {
      //Menu (other than level 1 menu item) or combo box has focus.
      //Do nothing, standard browser functionality should occur
   }
   else {
      if (negOpName == "null" && negEventName == "null") {
	      //If neither negOpName nor negEventName have been set in the jsp we first need to traverse
	      //the DOM to find the appropriate action buttons to set as default
      	negEventName = findDefaultButton(event, negEventName);
      }
      if (negEventName != "null" || negOpName != "null") {
			try {
				//if the field has an onblur event perform that before submitting page. 
				//This ensures any field formatting is carried out
				srcEl.onblur();
			}
			catch (ex) {
				//do nothing. If the field has no onblur event then it doesn't need formatting.
			}
			//domDebug(negOpName + negEventName);
         submitEventOrOp(negOpName, negEventName);
      }
   }
}

/**
/ findDefaultButton()
/
/ Traverses the DOM to find the appropriate action buttons to set as default
**/

function findDefaultButton(event, eventName) {
   var keyCode = (!keyhandler_isIE) ? event.which : event.keyCode;

   for (i=0; i < document.links.length; i++) {
      if (document.links[i].className == "actButton") {
         if (keyhandler_isIE) {
            var txt = document.links[i].innerText;
         }
         else {
            var txt = document.links[i].text;
         }
         //remove any whitespace from the text string
         txt = txt.replace(/\s/g, "");
         //If a hotkey is included in the action button text remove this from txt.
         //This is necessary since the text may or may not include a hotkey.
         var lastChar = txt.substring(txt.length-1,txt.length);
         if (lastChar == ")") {
            txt = txt.substring(0,txt.length-3);
         }
         if (keyCode == keyhandler_enter) {
	         //default positive action button is required
	         if (txt == "OK") {
	  	         eventName = document.links[i].id;
      	      break;
         	}
         }
			else if (keyCode == keyhandler_escape) {
	         //default negative action button is required
            if (txt == "Exit" || txt == "Exitwithoutsaving" || txt == "Cancel") {
               eventName = document.links[i].id;
               break;
            }			
			}
      }
   }
   
   return eventName;  
}

/**
/ stopBeep()
/
/ Function to stop IE beeping when the enter or return keys are pressed. Only happens when focus is
/ on tables around certain controls.
**/

function stopBeep(event) {
   // the beeping only occurs in IE
   if (keyhandler_isIE) {
      var srcEl = event.srcElement;
      var tag = srcEl.tagName;
      var type = srcEl.type;

      //focus is on a table containing a textarea, menu, action button, combo box or list box
      if ((tag == "TEXTAREA") || (tag == "A")  || (tag == "SELECT") 
               //focus is on a table containing an input field and a key other 
               //than return or enter has been pressed
               || (tag == "INPUT" && !(event.keyCode == 13))) {
         //Do nothing, standard browser functionality should occur
      }
      else {
         //focus is on any other control or on the window. Return false to stop the beep
         event.returnValue = false;
      }
   }
}
