// windowhandler.js
//
// This JavaScript provides the functionality for the desktop window handling
// relating to the currently open windows/pages.
//
// This includes:
// * Maintaining an array of windows that are or have been opened
// * Population of the desktop list box detailing currently open windows
//

var pageId              = 0;
var noOpenWindowsMessage = "There are no open windows";   // List box message when there are no open window
var KEY_LINE_NAME       = "displayedKeyValue";           //
var pageOpeningInUse    = false;                         // Is the pageOpening function currently in use
var pageClosingInUse    = false;                         // Is the pageClosing function currently in use

// Holds the reference for all open windows.
window.applicationWindows = new Object();

// Misc. global variables also used by other JS - need moving to a utility JS file.
var dsk_isIE = (!document.all) ? 0 : 1;
var _console = null;

// ****************************************************************************
// windowDescription
// Class to set up a window description
// ****************************************************************************
function windowDescription(winObj, title) {
   this.winObj    = winObj;                              // Window id returned when the window was opened
   this.title     = title;                               // Page title as define dby the <title> tag
   this.id        = pageId;                              // A unique page id
   this.ix        = -1;                                  // Current desktop list box index of this window
   this.closed    = false;                               // Window status
   pageId++;
}

// ****************************************************************************
// ****************************************************************************
function displayListboxTitle() {
   var spacers = new String();
	var titleWidth;
	
   // if the browser is not IWB, add spaces
   if (navigator.userAgent.toLowerCase().indexOf("gecko") == -1) {
     	titleWidth = listBoxWidth+1;
   }
   else {
   	if (screen.availHeight == 1200) {
	   	titleWidth = listBoxWidth+2;
	   }
	   else {
	   	titleWidth = listBoxWidth+3;
	   }
   }
   
   for (n=0; n <= titleWidth - listBoxHeading.length; n++){
         spacers += "&nbsp;";
   }
   
   document.writeln(listBoxHeading + spacers + "\n");
}

// ****************************************************************************
// registerOpeningPage
// Calls the pageOpening function
// ****************************************************************************
function registerOpeningPage(titleText) {
   if (opener != null && !opener.closed) {
      opener.pageOpening(titleText, window);
   }
}

// ****************************************************************************
// registerClosingPage
// Calls the windowClosing function
// ****************************************************************************
function registerClosingPage() {
   if (opener != null && !opener.closed) {
      opener.pageClosing(window);
   }
}

// ****************************************************************************
// pageOpening
// Saves information on the opening window and manages the open windows 
// listbox when an application window is opened.
// ****************************************************************************
function pageOpening(titleText, win) {
	var openWinsListbox = window.document.forms["openWindows"].elements["openWindowList"];

   // Flag this function as being in use
   pageOpeningInUse = true;

   ////domDebug("pageOpening " + win.name);

   // Ensure that all spaces in the middle of the string are replaced with hard spaces
   titleText = titleText.replace(/ /g,"\xA0");

   // Ensure the text is the correct length
   if (titleText.length < listBoxWidth) {
      for (var i=titleText.length; i<listBoxWidth; i++) {
         titleText = titleText + "\xA0";
      }
   }
   else if (titleText.length > listBoxWidth) {
      titleText = titleText.substring(0, listBoxWidth);
   }

	//if there were no windows open previously we first need to remove the "no windows"
	//message from the listbox
	if (getApplicationWinsLength() == 0) {
		openWinsListbox.length = 0;
	}

	//create a new window description object
	newWinDesc = new windowDescription(win, titleText);
	
   //add the new window description object to the applicationWindows array
   window.applicationWindows["window" + win.name] = newWinDesc;    	  
  
	//add a new entry to the listbox for the window that has just been opened  
	updateListbox();
	
	//highlight the listbox entry for the window that has just been opened
	for (i=0; i<openWinsListbox.options.length; i++) {
		if (openWinsListbox.options[i].id == win.name) {
			openWinsListbox.selectedIndex = i;
			break;			
		}
	}

   pageOpeningInUse = false;
}

// ****************************************************************************
// pageClosing
// Manages the open windows listbox when an application window is closed.
// ****************************************************************************
function pageClosing(win) {
	var openWinsListbox = window.document.forms["openWindows"].elements["openWindowList"];
	var selected = openWinsListbox.selectedIndex;
	var closingWinIndex;
	
	if (selected != -1) {
		var currentSelId = openWinsListbox.options[selected].id;
	}
	else {
		//no entry is currently selected so choose the first entry
		var currentSelId = openWinsListbox.options[0].id;
	}		
	
	//find the index in the listbox of the window that is closing
	for (i=0; i<openWinsListbox.options.length; i++) {
		if (openWinsListbox.options[i].id == win.name) {
			closingWinIndex = i;
			break;			
		}
	}

	//remove the entry in the applicationWindows array for the window that has just been closed
	delete window.applicationWindows["window" + win.name];
	
	//Provided the windows have not been closed using close all remove the entry in the listbox 
	//for the window that has just been closed. If close all has been used close all processing 
	//will call updateListbox()
	if (win.document.forms["HAFForm"].elements["HAFOperator.closeAllFinished"].value == "false") {
		updateListbox();
	}
	
	//find which entry in the listbox to highlight
	if (closingWinIndex == selected) {
		//if the closing window is the entry currently highlighted in the listbox, highlight 
		//the first entry in the listbox
		openWinsListbox.selectedIndex = 0;
	}
	else {
		//otherwise highlight should remain on the currently highlighted entry
		for (i=0; i<openWinsListbox.options.length; i++) {
			if (openWinsListbox.options[i].id == currentSelId) {
				openWinsListbox.selectedIndex = i;
				break;			
			}
		}
	}	
		 
   if (!dsk_isIE) {
      // Do this here for IWB, IE does it at after closing page in askToClose in closeall.js
      pageClosingInUse = false;
   }
 
}

// ****************************************************************************
// updateListbox
// Updates the open window list on the desktop after a window has been 
// opened or closed
// ****************************************************************************
function updateListbox() {
	var openWinsListbox = window.document.forms["openWindows"].elements["openWindowList"];
	var newOpt = new Option();
	
	openWinsListbox.options.length = 0;
	var i=0;
	for (apps in window.applicationWindows) {
		var newOpt = new Option();
		newOpt.id = window.applicationWindows[apps].winObj.name;
		newOpt.text = window.applicationWindows[apps].title;
		newOpt.className = i & 1 ? "listBoxAltMulti" : "fixedNormal";
		openWinsListbox.options[i] = newOpt;
		i++;
	}		
	
	//if the only listbox entry has been removed display the 'no open windows message'	
   checkForNoWindows();
   
	//does the maximum number of windows reached message need hiding or showing?   
   maxWindowsMessage();
}

// ****************************************************************************
// getApplicationWinsLength
// returns the length of the applicationWindows array, i.e. the number of 
// application windows currently open.
// ****************************************************************************
function getApplicationWinsLength() {
	var arrayLength = 0;
   
   if (isDesktop){
      win = window;
   }
   else {
      win = opener.window;
   }
   
	for (apps in win.applicationWindows) {
		arrayLength = arrayLength + 1;
	}
	
	return arrayLength;
}

// ****************************************************************************
// checkForNoWindows
// ****************************************************************************
function checkForNoWindows() {
   if (getApplicationWinsLength() == 0) {
   	//no windows are open so display "There are no open windows" entry in the open windows listbox
      displayNoWindowsMessage();
   }
}

// ****************************************************************************
// displayNoWindowsMessage
// Adds the "There are no open windows" entry to the open windows listbox.
// ****************************************************************************
function displayNoWindowsMessage() {
	var openWinsListbox = window.document.forms["openWindows"].elements["openWindowList"];

   // Construct the "no windows" message" and add spaces to make it the right length
   for (var i=noOpenWindowsMessage.length; i<listBoxWidth; i++) {
      noOpenWindowsMessage = noOpenWindowsMessage + "\xA0";
   }

   // Add the message to the desktop list box
   openWinsListbox.options[0] = new Option(noOpenWindowsMessage);
   openWinsListbox.options[0].className = "fixedNormal";
   openWinsListbox.selectedIndex = 0;

   // Delete the menuDescription array to stop it growing too large
   menuDescription.length = 0;
}

// ****************************************************************************
// switchWindow
// moves focus to the window which has been selected from the open windows
// listbox. The selection is made either by double clicking or pressing
// enter on an entry.
// ****************************************************************************
function switchWindow() {
	var openWinsListbox = window.document.forms["openWindows"].elements["openWindowList"];
   var selected = openWinsListbox.selectedIndex;
   
	for (apps in window.applicationWindows) {
		if (openWinsListbox.options[selected].id == window.applicationWindows[apps].winObj.name) {
			if (dsk_isIE) {
				window.applicationWindows[apps].winObj.document.focus();
				window.applicationWindows[apps].winObj.focus();
			}
			else {
				window.applicationWindows[apps].winObj.focus();
			}
			break;
		}
	}
}

// ****************************************************************************
// setOpenWindowListFocus
// puts focus on the correct entry in the open windows listbox when the F8 key
// is used to switch between desktop and application windows
// ****************************************************************************
function setOpenWindowListFocus(win) {
	var openWinsListbox = window.document.forms["openWindows"].elements["openWindowList"];
   window.focus();
   openWinsListbox.focus();
   
   //whichever application window F8 was pressed on, highlight it's entry in the open windows listbox
	for (i=0; i<openWinsListbox.options.length; i++) {
		if (openWinsListbox.options[i].id == win.name) {
			openWinsListbox.selectedIndex = i;
			break;			
		}
	} 
}

// ****************************************************************************
// maxWindowsMessage
// If the number of windows open equals the max number of app windows allowed 
// (defined in desktop.js), a warning message is displayed on the desktop. If 
// this message is currently displayed and the number of windows open is less
// than the max allowed the message is removed. 
// ****************************************************************************
function maxWindowsMessage() {
   var theDiv = window.document.getElementById('maxAppDiv');
   if (getApplicationWinsLength() >= desktop_maxWindows) {
      var str = "";
      // create html warning message
      str += ("<table>\n");
      str += ("   <tr>\n");
      str += ("      <td><img "+warningImg+">&nbsp;</td>\n");
      str += ("      <td><span class=\"varNormal\">You now have " + getApplicationWinsLength() + " application windows open which is the system's maximum.</span></td>\n");
      str += ("   </tr>\n");
      str += ("   <tr>\n");
      str += ("      <td></td>\n");   
      str += ("      <td><span class=\"varNormal\">Before launching another, you must first close or complete one or more of the open applications.</span></td>\n");
      str += ("   </tr>\n");
      str += ("</table>\n");
      theDiv.innerHTML= str;
   }
   else{
      theDiv.innerHTML= "";
   }
}
   

// ****************************************************************************
// ****************************************************************************
function debugShowWins() {
   if (windowDesc.length == 0) {
      ////domDebug("  *** no open windows ***");
   }

   for (i=0; i<windowDesc.length; i++) {
      ////domDebug("  " + i + ", ix= " + windowDesc[i].ix + " (" + windowDesc[i].closed + ") " + windowDesc[i].title);
   }
}

// ****************************************************************************
// ****************************************************************************
function domDebug(msg){
   if (opener != null && !opener.closed && opener.document.forms[0].name == "DesktopForm") {
      opener.domDebug(msg);
   }
   else {
      if ((_console == null) || (_console.closed)) {
         _console = window.open("","console","width=600,height=300,resizable,scrollbars");
         _console.document.open("text/html");
         _console.document.writeln("<pre>");
      }

      now = new Date();
      zeroes = ".00"
      ms = now.getMilliseconds();
      ms = zeroes.substr(0, 3 - ms.length) + ms;
      if (ms.length == 2) { ms = "0" + ms; }
      if (ms.length == 1) { ms = "00" + ms; }

      _console.document.writeln(now.toUTCString().substr(17, 8) + ms.length + " : " + msg);
   }
}
